@@ -18,9 +18,8 @@
 #define myx_admin_public_interface_h
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
-#define __LCC__
+#include <winsock2.h>
 #endif
-
 #include <stdio.h>
 #include <mysql.h>
 #include <pcre.h>
@@ -83,19 +82,21 @@ typedef enum myx_rbs_options
 
 typedef enum myx_backup_options
 {
-  MYX_B_NO_CREATES=           1,
-  MYX_B_NO_EXTENDED_INSERT=   2,
-  MYX_B_ADD_DROP_TABLE=       4,
-  MYX_B_COMMENT=              8,
-  MYX_B_DONT_WRITE_FULL_PATH= 16,
-  MYX_B_LOCK_ALL_TABLES=      32,
-  MYX_B_SINGLE_TRANSACTION=   64,
-  MYX_B_DISABLE_KEYS=         128,
-  MYX_B_COMPLETE_INSERTS=     256,
-  MYX_B_ANSI_QUOTES=          512,
-  MYX_B_ADD_LOCKS=            1024,
-  MYX_B_COMPLETE_SCHEMATAS=   2048,
-  MYX_B_SORT_TABLES=          4096
+  MYX_B_NO_CREATES=               1,
+  MYX_B_NO_EXTENDED_INSERT=       2,
+  MYX_B_ADD_DROP_TABLE=           4,
+  MYX_B_COMMENT=                  8,
+  MYX_B_DONT_WRITE_FULL_PATH=    16,
+  MYX_B_LOCK_ALL_TABLES=         32,
+  MYX_B_SINGLE_TRANSACTION=      64,
+  MYX_B_DISABLE_KEYS=           128,
+  MYX_B_COMPLETE_INSERTS=       256,
+  MYX_B_ANSI_QUOTES=            512,
+  MYX_B_ADD_LOCKS=              1024,
+  MYX_B_COMPLETE_SCHEMATAS=     2048,
+  MYX_B_SORT_TABLES=            4096,
+  MYX_B_COMPATIBILITY_MODE=     8192,
+  MYX_B_POINT_IN_TIME_BACKUP = 16384
 } MYX_BACKUP_OPTIONS;
 
 typedef enum myx_backup_error
@@ -589,6 +590,7 @@ MYX_PUBLIC_FUNC MYX_GUI_DESCRIPTION * myx_get_gui_description(const char *filena
 MYX_PUBLIC_FUNC int myx_free_gui_description(MYX_GUI_DESCRIPTION *gui_desc);
 
 MYX_PUBLIC_FUNC MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc, const char *ini_filepath, const char *group);
+MYX_PUBLIC_FUNC MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_filef(MYX_GUI_DESCRIPTION *desc, const char *ini_filepath, FILE *new_file, const char *group);
 MYX_PUBLIC_FUNC MYX_STRINGLIST* myx_get_all_cnf_sections(const char *filename, MYX_ADMIN_LIB_ERROR *error_code);
 MYX_PUBLIC_FUNC char* myx_get_cnf_value(const char *filename, const char *section, const char *name, MYX_ADMIN_LIB_ERROR *error_code);
 
@@ -618,6 +620,8 @@ MYX_PUBLIC_FUNC MYX_BACKUP_ERROR
                        void *ruser_data, void (*report_warning) (const char *msg, void *user_data), void *wuser_data);
 
 
+MYX_PUBLIC_FUNC char * myx_get_backup_error_string(MYX_BACKUP_ERROR error);
+
 /* Expression evaluation functions */
 MYX_PUBLIC_FUNC MYX_COMPILED_EXPRESSION * myx_compile_expression(const char *expr, MYX_EXPRESSION_ERROR *error);
 MYX_PUBLIC_FUNC double myx_eval_expression(MYX_COMPILED_EXPRESSION *expr, MYX_VARIABLES *old_vars, MYX_VARIABLES *vars, MYX_EXPRESSION_ERROR *error);
@@ -627,6 +631,11 @@ MYX_PUBLIC_FUNC int myx_free_expression(MYX_COMPILED_EXPRESSION *expr);
 MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block_num, int *last_block);
 MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_num, int *last_block);
 MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int block_num, int *last_block);
+
+MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_error_logf(FILE *fp, int block_size, int block_num, int *last_block);
+MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_slow_logf(FILE *fp, int block_size, int block_num, int *last_block);
+MYX_PUBLIC_FUNC MYX_LOGFILE* myx_parse_general_logf(FILE *fp, int block_size, int block_num, int *last_block);
+
 MYX_PUBLIC_FUNC int myx_free_logfile(MYX_LOGFILE *logfile);
 
 MYX_PUBLIC_FUNC char* myx_get_errorlog_filename(MYSQL *mysql);
@@ -639,9 +648,9 @@ MYX_PUBLIC_FUNC MYX_TABLE_COMMAND_STATUSES * myx_repair_table(MYSQL *mysql, cons
 MYX_PUBLIC_FUNC int myx_free_command_status(MYX_TABLE_COMMAND_STATUSES *status);
  
 /* Health xml functions */
-MYX_PUBLIC_FUNC MYX_HEALTH_PAGES* myx_read_in_health_pages(char* filename, MYX_ADMIN_LIB_ERROR *error_code);
+MYX_PUBLIC_FUNC MYX_HEALTH_PAGES* myx_read_in_health_pages(const char* filename, MYX_ADMIN_LIB_ERROR *error_code);
 MYX_PUBLIC_FUNC int myx_free_health_pages(MYX_HEALTH_PAGES *pages);
-MYX_PUBLIC_FUNC int myx_save_health_pages(MYX_HEALTH_PAGES *pages, char *filename);
+MYX_PUBLIC_FUNC int myx_save_health_pages(MYX_HEALTH_PAGES *pages, const char *filename);
 
 /* replication functions */
 MYX_PUBLIC_FUNC MYX_REPL_HOSTS * myx_show_repl_hosts_status(MYSQL *mysql, MYX_USER_REPL_HOSTS *replist, MYX_ADMIN_LIB_ERROR *error_code);
