@@ -226,10 +226,14 @@ MYX_VARIABLES* myx_get_server_variables(MYSQL *mysql)
 {
   MYX_VARIABLES *vars;
 
+  /*if (mysql_full_version_is_later_or_equal_than(mysql, 4, 0, 3))
+    vars= myx_get_variables(mysql, "SHOW GLOBAL VARIABLES");
+  else*/
   vars= myx_get_variables(mysql, "SHOW VARIABLES");
 
   
-  vars= add_missing_server_variables(mysql, vars);
+  if (vars)
+    vars= add_missing_server_variables(mysql, vars);
 
   return vars;
 }
@@ -238,9 +242,13 @@ MYX_VARIABLES* myx_get_status_variables(MYSQL *mysql)
 {
   MYX_VARIABLES *vars;
 
-  vars= myx_get_variables(mysql, "SHOW STATUS");
+  if (mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 2))
+    vars= myx_get_variables(mysql, "SHOW GLOBAL STATUS");
+  else
+    vars= myx_get_variables(mysql, "SHOW STATUS");
 
-  vars= add_innodb_status(mysql, vars);
+  if (vars)
+    vars= add_innodb_status(mysql, vars);
 
   return vars;
 }
@@ -661,11 +669,11 @@ double myx_eval_expression(MYX_COMPILED_EXPRESSION *expr,
       break;
     case 'D':
       {
-        double tmp1, tmp2;
-        tmp2= strtod((char*)vars->variables[expr->nodes[i].variable_index].value, NULL);
-        tmp1= strtod((char*)old_vars->variables[expr->nodes[i].variable_index].value, NULL);
-        if (tmp1 > tmp2)
-          stack[stp++]= tmp2-tmp1;
+        double nval, oval;
+        nval= strtod((char*)vars->variables[expr->nodes[i].variable_index].value, NULL);
+        oval= strtod((char*)old_vars->variables[expr->nodes[i].variable_index].value, NULL);
+        if (oval < nval)
+          stack[stp++]= nval-oval;
         else
           stack[stp++]= 0.0;
       }
