@@ -17,7 +17,7 @@
 #define BUFFER_LEN 256 //used by get_backup_content
 #define DEFAULT_SCHEMA_NAME "DEFAULT_SCHEMA"
 #define DEFAULT_CATALOG_NAME "DEFAULT_CATALOG"
-#define ADMIN_DUMP_VERSION "1.3"
+#define ADMIN_DUMP_VERSION "1.4"
 
 #define INITIAL_BUFFER_LENGTH 1024*1024 /* initial size of 1M */
 
@@ -34,12 +34,14 @@
 #include "myx_library.h"
 #include <errno.h>
 #include <ctype.h>
-#include <glib.h>
+#include <glib/gprintf.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 #include <time.h>
 
+//XXX add gettext support
+#define _(s) s
 
 /* structure declarations */
 
@@ -251,7 +253,7 @@ MYX_ADMIN_LIB_ERROR myx_save_profile(const char *profile_name,
     }
   }
 
-  filename= g_strconcat(profile_directory, profile_name, NULL);
+  filename= g_build_path("/", profile_directory, profile_name, NULL); // Don't forget the NULL parameter to finish the parameter list!
   res= myx_xmlSaveFile(filename, doc);
   g_free(filename);
   xmlFreeDoc(doc);
@@ -553,7 +555,15 @@ int myx_free_bs_status(MYX_BS_STATUS *b)
   return 0;
 }
 
-
+static const char * show_master_status_fields[]=
+{
+  "File",               // 0
+  "Position",           // 1
+  "Binlog_Do_DB",       // 2
+  "Binlog_Ignore_DB"    // 3
+};
+static const char ** show_master_status_fields_end=
+             show_master_status_fields + sizeof(show_master_status_fields)/sizeof(char*);
 
 /*
  *----------------------------------------------------------------------
@@ -573,7 +583,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
                                             unsigned int num_rows,
                                             MYX_BACKUP_ERROR *error)
 {
-  MYSQL_ROW row;
+  MYSQL_ROW row= NULL;
 
   /* if we don't have any more tables to back up.. */
   if ( status->table_finished && 
@@ -591,12 +601,24 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
 
     if (status->starting)
     {
-      const char * sql_mode_cmd= (status->options & MYX_B_ANSI_QUOTES)
-           ? "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, sql_mode='ANSI_QUOTES' */"
-           : "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, sql_mode='' */";
+      gchar sql_mode_cmd[200] = "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, sql_mode='";
+      int mode_entry_count = 0;
+      if (status->options & MYX_B_ANSI_QUOTES)
+      {
+        mode_entry_count++;
+        g_sprintf(sql_mode_cmd, "%sANSI_QUOTES", sql_mode_cmd);
+      };
+      if (status->options & MYX_B_COMPATIBILITY_MODE)
+      {
+        g_sprintf(sql_mode_cmd, "%s%sMYSQL323", sql_mode_cmd, (mode_entry_count > 0) ? "," : "");
+        mode_entry_count++;
+      };
+      g_sprintf(sql_mode_cmd, "%s' */", sql_mode_cmd);
+
       if (write_sql_file_header(status, error))
         return -1;
 
+      // Backup method
       status->starting=0;
       if (status->options & MYX_B_LOCK_ALL_TABLES)
       {
@@ -606,12 +628,120 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
           return -1; /* fatal */
         }
       }
-      else if ((status->options & MYX_B_SINGLE_TRANSACTION) &&
-               myx_mysql_query(status->mysql, "BEGIN"))
+      else if (status->options & MYX_B_SINGLE_TRANSACTION)
       {
-        *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
-        return -1;  /* fatal */
+        // make sure the correct ISOLATION LEVEL is set
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5) == 0)
+        {
+          if (myx_mysql_query(status->mysql, "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
+        }
+
+        // start the transaction
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 1, 8) == 0)
+        {
+          if (myx_mysql_query(status->mysql, "START TRANSACTION WITH CONSISTENT SNAPSHOT"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
+
+
+        }
+        else
+        {
+          if (myx_mysql_query(status->mysql, "BEGIN"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
+        }
       }
+      else if (status->options & MYX_B_POINT_IN_TIME_BACKUP)
+      {
+        if (myx_mysql_query(status->mysql, "FLUSH TABLES"))
+        {
+          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+          return -1;  /* fatal */
+        }
+        if (myx_mysql_query(status->mysql, "FLUSH TABLES WITH READ LOCK"))
+        {
+          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+          return -1;  /* fatal */
+        }
+        // make sure the correct ISOLATION LEVEL is set
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5) == 0)
+        {
+          if (myx_mysql_query(status->mysql, "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
+        }
+        if (myx_mysql_query(status->mysql, "START TRANSACTION WITH CONSISTENT SNAPSHOT"))
+        {
+          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+          return -1;  /* fatal */
+        }
+        if (myx_mysql_query(status->mysql, "SHOW MASTER STATUS"))
+        {
+          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+          return -1;  /* fatal */
+        }
+        else
+        {
+          MYSQL_RES *res;
+
+          if (res= mysql_store_result(status->mysql))
+          {
+            MYSQL_ROW row;
+            MYSQL_FIELD *fields;
+            int num_fields;
+            int fi[5];
+            char *binlog_filename= NULL;
+            char *binlog_pos= NULL;
+
+            
+            //Depending on the version of the server there might be different columns
+            num_fields= mysql_num_fields(res);
+            fields= mysql_fetch_fields(res);
+
+            build_field_subst(show_master_status_fields, show_master_status_fields_end,
+              fields, fields + num_fields, fi);
+
+            if (row= mysql_fetch_row(res))
+            {
+              if (fi[0] > -1)
+                binlog_filename= myx_convert_dbstr_utf8(status->mysql, row[fi[0]]);
+              if (fi[1] > -1)
+                binlog_pos= g_strdup(row[fi[1]]);
+            }
+
+            if (binlog_filename && binlog_pos)
+            {
+              SAFE_IO(fprintf(status->sql_file,
+                "--\n"
+                "-- Position to start replication or point-in-time recovery from\n"
+                "--\n\n"
+                "-- CHANGE MASTER TO MASTER_LOG_FILE='%s', MASTER_LOG_POS=%s;\n\n",
+                binlog_filename, binlog_pos));
+            }
+
+            mysql_free_result(res);
+          }
+        }
+
+        if (myx_mysql_query(status->mysql, "UNLOCK TABLES"))
+        {
+          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+          return -1;  /* fatal */
+        }
+      }
+
+
       if (myx_mysql_query(status->mysql,sql_mode_cmd))
       {
         *error= MYX_BACKUP_CANNOT_SET_ANSI_QUOTES;
@@ -673,6 +803,15 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
       return -1;
     }
 
+    if (status->options & MYX_B_COMMENT)
+    {
+      SAFE_IO(fprintf(status->sql_file, "\n--\n"
+        "-- Dumping data for table %s.%s\n"
+        "--\n", 
+        status->current_schema_quoted,
+        status->current_table_quoted));
+    }
+
     /* get the rows of the new table */
     sel_stmt= g_strconcat("SELECT /*!40001 SQL_NO_CACHE */ * FROM ",
                           status->current_schema_quoted, ".",
@@ -746,7 +885,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
     if (status->options &  MYX_B_DISABLE_KEYS)
     {
       SAFE_IO(fprintf(status->sql_file,
-                      "\n/*!40000 ALTER TABLE %s ENABLE KEYS */;\n",
+                      "/*!40000 ALTER TABLE %s ENABLE KEYS */;\n\n",
                       status->current_table_quoted));
     }
 
@@ -761,8 +900,10 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
           return -1;
         }
       }
-      else if ((status->options & MYX_B_SINGLE_TRANSACTION) &&
-                myx_mysql_query(status->mysql, "COMMIT"))
+      else if ((
+        (status->options & MYX_B_SINGLE_TRANSACTION) ||
+        (status->options & MYX_B_POINT_IN_TIME_BACKUP)) &&
+        myx_mysql_query(status->mysql, "COMMIT"))
       {
         *error= MYX_BACKUP_MYSQL_ERROR;
         return -1;  /* fatal */
@@ -1496,6 +1637,10 @@ static int write_path_info_to_file(MYX_BS_STATUS *status,
     if (myx_mysql_query(status->mysql, stmt) ||
         !(result= mysql_store_result(status->mysql)) )
     {
+      SAFE_IO(fprintf(status->sql_file, "--\n"
+        "-- Create schema %s\n"
+        "--\n\n", status->current_schema_quoted));
+
       SAFE_IO(fprintf(status->sql_file,
                       "CREATE DATABASE /*!32312 IF NOT EXISTS*/ %s;\n",
                       status->current_schema_quoted));
@@ -1523,10 +1668,12 @@ static int write_path_info_to_file(MYX_BS_STATUS *status,
 static int write_sql_file_header(MYX_BS_STATUS *status,
                                  MYX_BACKUP_ERROR *error)
 {
+  gchar Buffer[200] = "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO";
+
   if (status->options & MYX_B_COMMENT)
   {
     SAFE_IO(fprintf(status->sql_file,
-            "-- MySQL-Administrator dump %s\n--\n", ADMIN_DUMP_VERSION));
+            "-- MySQL Administrator dump %s\n--\n", ADMIN_DUMP_VERSION));
     SAFE_IO(fputs("-- ------------------------------------------------------\n",
                   status->sql_file));
     SAFE_IO(fprintf(status->sql_file,
@@ -1535,20 +1682,20 @@ static int write_sql_file_header(MYX_BS_STATUS *status,
     SAFE_IO(fprintf(status->sql_file, "\n\n"));
   }
 
-  SAFE_IO(fprintf(status->sql_file,"SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT;\n"));
-  SAFE_IO(fprintf(status->sql_file,"SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS;\n"));
-  SAFE_IO(fprintf(status->sql_file, "SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION;\n"));
-  SAFE_IO(fprintf(status->sql_file, "SET NAMES %s;\n\n", "utf8"));
-  SAFE_IO(fprintf(status->sql_file,"/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;\n"));
-  SAFE_IO(fprintf(status->sql_file,"/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET NAMES %s */;\n\n", "utf8"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;\n"));
+
   if (status->options & MYX_B_ANSI_QUOTES)
-  {
-    SAFE_IO(fprintf(status->sql_file,"/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='ANSI_QUOTES,NO_AUTO_VALUE_ON_ZERO' */;\n\n\n"))
-  }
-  else
-  {
-    SAFE_IO(fprintf(status->sql_file,"/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE=NO_AUTO_VALUE_ON_ZERO */;\n\n\n"))
-  }
+    g_sprintf(Buffer, "%s, ANSI_QUOTES", Buffer);
+  if (status->options & MYX_B_COMPATIBILITY_MODE)
+    g_sprintf(Buffer, "%s, MYSQL323", Buffer);
+  g_sprintf(Buffer, "%s' */;\n\n\n", Buffer);
+  SAFE_IO(fprintf(status->sql_file, Buffer));
+
   return 0;
 }
 
@@ -1558,9 +1705,9 @@ static int write_sql_file_footer(MYX_BS_STATUS *status, MYX_BACKUP_ERROR *error)
   SAFE_IO(fprintf(status->sql_file,"/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;\n"));
   SAFE_IO(fprintf(status->sql_file,"/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;\n"));
 
-  SAFE_IO(fprintf(status->sql_file, "SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT;\n"));
-  SAFE_IO(fprintf(status->sql_file, "SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS;\n"));
-  SAFE_IO(fprintf(status->sql_file, "SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;\n"));
+  SAFE_IO(fprintf(status->sql_file, "/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;\n"));
 
   fprintf(status->sql_file,"/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\n");
 
@@ -1581,6 +1728,15 @@ static int write_create_statement_to_file(MYX_BS_STATUS *status,
     return -1;
   }
 
+  if (status->options & MYX_B_COMMENT)
+  {
+    SAFE_IO(fprintf(status->sql_file, "\n--\n"
+      "-- Table structure for table %s.%s\n"
+      "--\n", 
+      status->current_schema_quoted,
+      status->current_table_quoted));
+  }
+
   stmt= g_strconcat("SHOW CREATE TABLE ",
                     status->current_schema_quoted, ".",
                     status->current_table_quoted, NULL);
@@ -2061,3 +2217,33 @@ MYX_BACKUP_CONTENT * myx_get_restore_drop_list(MYSQL *mysql, MYX_BACKUP_CONTENT
   
   return drop_list;
 }
+
+
+char * myx_get_backup_error_string(MYX_BACKUP_ERROR error)
+{
+  switch (error)
+  {
+  case MYX_BACKUP_NO_ERROR:
+    return g_strdup(_("Success."));
+  case MYX_BACKUP_MYSQL_ERROR:
+    return g_strdup(_("MySQL Error."));
+  case MYX_BACKUP_CANT_OPEN_FILE:
+    return g_strdup(_("Cannot open file."));
+  case MYX_BACKUP_ILLEGAL_OPTION:
+    return g_strdup(_("Illegal option."));
+  case MYX_BACKUP_PCRE_ERROR:
+    return g_strdup(_("Internal parsing error (pcre)."));
+  case MYX_BACKUP_MALLOC_FAILED:
+    return g_strdup(_("Memory allocation failed."));
+  case MYX_BACKUP_OUTPUTDEVICE_FULL:
+    return g_strdup(_("Disk full."));
+  case MYX_BACKUP_CANNOT_FLUSH_TABLES_WITH_READ_LOCK:
+    return g_strdup(_("Cannot flush tables with read lock."));
+  case MYX_BACKUP_CANNOT_START_TRANSACTION:
+    return g_strdup(_("Cannot start transaction."));
+  case MYX_BACKUP_CANNOT_SET_ANSI_QUOTES:
+    return g_strdup(_("Cannot set ANSI quotes."));
+  }
+  return NULL;
+}
+
