@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <time.h>
 #include <assert.h>
+#include <sys/stat.h>
 #include <glib.h>
 #include <pcre.h>
 #include "myx_xml_aux_functions.h"
@@ -143,10 +144,38 @@ static int find_line_beginning(FILE *fp, bigint *offset, bigint file_size)
 }
 
 
+static size_t get_fsize(FILE *file)
+{
+  struct stat st;
+  
+  if (fstat(fileno(file), &st) < 0)
+	return 0;
+  else
+	return st.st_size;
+}
+
+
+
 MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int block_num, int *last_block)
 {
-  char line[LOG_FILE_LINE_LEN];
   FILE *fp;
+  MYX_LOGFILE *logfile;
+  assert(filename != NULL);
+  
+  if (! (fp= myx_fopen(filename, "r")) )
+    return NULL;
+  
+  logfile= myx_parse_general_logf(fp, block_size, block_num, last_block);
+
+  fclose(fp);
+  
+  return logfile;
+}
+
+
+MYX_LOGFILE* myx_parse_general_logf(FILE *fp, int block_size, int block_num, int *last_block)
+{
+  char line[LOG_FILE_LINE_LEN];
   bigint offset;
   int line_no;
   pcre *re_connect, *re_init, *re_quit, *re_query;
@@ -158,19 +187,15 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
   bigint bytes_read;
   bigint file_size= -1;
   
-  assert(filename != NULL);
   assert(block_size > 0);
   assert(block_num != 0);
   assert(last_block != NULL);
   
-  if (! (fp= myx_fopen(filename, "r")) )
-  {
-    return NULL;
-  }
+  rewind(fp);
   
   { /* seek to correct point (we need to begin after a newline) */
     int desired_seek_point= block_size * block_num;
-    file_size= get_file_size(filename);
+    file_size= get_fsize(fp);
     offset=  (desired_seek_point > file_size) ? file_size: desired_seek_point;
 
     offset *= -1;
@@ -187,7 +212,6 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
   
   if (fseek(fp, (long)offset, SEEK_END) )
   {
-    fclose(fp);
     return NULL;
   }
 
@@ -200,8 +224,7 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
   
   if (!re_connect || !re_init || !re_quit || !re_query)
   {
-    fclose(fp);
-    return NULL;
+     return NULL;
   }
   
   logfile= g_malloc0(sizeof(MYX_LOGFILE));
@@ -251,8 +274,6 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
       ev_type= MYX_EVENT_QUERY;
     }
 
-
-    
     if (ev_type > 0) 
     {
       MYX_LOGFILE_EVENT *event;
@@ -281,11 +302,10 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
   
   if (ferror(fp))
   {
-    fclose(fp);
+	myx_free_logfile(logfile);
     return NULL;
   }
   
-  fclose(fp);
   return logfile;  
 }
 
@@ -293,8 +313,25 @@ MYX_LOGFILE* myx_parse_general_log(const char *filename, int block_size, int blo
 
 MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_num, int *last_block)
 {
-  char line[LOG_FILE_LINE_LEN];
   FILE *fp;
+  MYX_LOGFILE *logfile;
+  
+  assert(filename != NULL);
+
+  if (! (fp= myx_fopen(filename, "r")) )
+    return NULL;
+  
+  logfile= myx_parse_slow_logf(fp, block_size, block_num, last_block);
+  
+  fclose(fp);
+  
+  return logfile;
+}
+
+
+MYX_LOGFILE* myx_parse_slow_logf(FILE *fp, int block_size, int block_num, int *last_block)
+{
+  char line[LOG_FILE_LINE_LEN];
   bigint offset;
   int line_no;
   pcre *re_select;
@@ -306,27 +343,21 @@ MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_
   bigint bytes_read;
   bigint file_size= -1;
   
-  assert(filename != NULL);
   assert(block_size > 0);
   assert(block_num != 0);
   assert(last_block != NULL);
   
-  if (! (fp= myx_fopen(filename, "r")) )
-  {
-    return NULL;
-  }
+  rewind(fp);
   
   { /* seek to correct point (we need to begin after a newline) */
     int desired_seek_point= block_size * block_num;
-    file_size= get_file_size(filename);
+    file_size= get_fsize(fp);
     offset=  (desired_seek_point > file_size) ? file_size: desired_seek_point;
 
     offset *= -1;
   
-  
     if (find_line_beginning(fp,&offset, file_size))
     {
-      fclose (fp);
       return NULL;
     }
     
@@ -335,7 +366,6 @@ MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_
   
   if (fseek(fp, (long)offset, SEEK_END) )
   {
-    fclose(fp);
     return NULL;
   }
 
@@ -345,7 +375,6 @@ MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_
 
   if (!re_select)
   {
-    fclose(fp);
     return NULL;
   }
   
@@ -392,11 +421,10 @@ MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_
   
   if (ferror(fp))
   {
-    fclose(fp);
+	myx_free_logfile(logfile);
     return NULL;
   }
   
-  fclose(fp);
   return logfile;  
 }
 
@@ -431,8 +459,25 @@ MYX_LOGFILE* myx_parse_slow_log(const char *filename, int block_size, int block_
  */
 MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block_num, int *last_block)
 {
-  char line[LOG_FILE_LINE_LEN];
   FILE *fp;
+  MYX_LOGFILE *logfile;
+  
+  assert(filename != NULL);
+  
+  if (! (fp= myx_fopen(filename, "r")) )
+    return NULL;
+  
+  logfile= myx_parse_error_logf(fp, block_size, block_num, last_block);
+  
+  fclose(fp);
+  
+  return logfile;
+}
+
+
+MYX_LOGFILE* myx_parse_error_logf(FILE *fp, int block_size, int block_num, int *last_block)
+{
+  char line[LOG_FILE_LINE_LEN];
   bigint offset;
   int line_no;
   pcre *re_mysql_start, *re_mysql_end, *re_mysql_error, *re_innodb_start, *re_innodb_shutdown, 
@@ -445,28 +490,22 @@ MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block
   bigint bytes_read;
   bigint file_size= -1;
   
-  assert(filename != NULL);
   assert(block_size > 0);
   assert(block_num != 0);
   assert(last_block != NULL);
-  
-  if (! (fp= myx_fopen(filename, "r")) )
-  {
-    return NULL;
-  }
+
+  rewind(fp);
   
   { /* seek to correct point (we need to begin after a newline) */
     int desired_seek_point= block_size * block_num;
-    file_size= get_file_size(filename);
+    file_size= get_fsize(fp);
     offset=  (desired_seek_point > file_size) ? file_size: desired_seek_point;
 
     offset *= -1;
   
-  
     if (find_line_beginning(fp,&offset, file_size))
     {
-      fclose (fp);
-      return NULL;
+	  return NULL;
     }
     
     *last_block= (offset == -file_size) ? 1 : 0;
@@ -474,7 +513,6 @@ MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block
   
   if (fseek(fp, (long)offset, SEEK_END) )
   {
-    fclose(fp);
     return NULL;
   }
 
@@ -491,7 +529,6 @@ MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block
   
   if (!re_mysql_start || !re_mysql_end || !re_mysql_error)
   {
-    fclose(fp);
     return NULL;
   }
   
@@ -595,11 +632,10 @@ MYX_LOGFILE* myx_parse_error_log(const char *filename, int block_size, int block
   
   if (ferror(fp))
   {
-    fclose(fp);
+	myx_free_logfile(logfile);
     return NULL;
-  }
-  
-  fclose(fp);
+  }  
+
   return logfile;
 }
 
