@@ -420,6 +420,88 @@ void safe_fprintf_TEXTEDIT(FILE * file, MYX_GUI_WIDGET * widget)
 MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
                                               const char *ini_filepath,
                                               const char *group)
+{
+  FILE *new_file;
+  char *new_ini_filepath;
+  char *bak_filepath;
+  MYX_ADMIN_LIB_ERROR error;
+  
+  new_ini_filepath= g_strdup_printf("%s.new", ini_filepath);
+  bak_filepath= g_strdup_printf("%s.old", ini_filepath);
+  
+  if ( !(new_file= myx_fopen(new_ini_filepath, "w")) )
+  {
+    fprintf(stderr,
+            "Error in function myx_update_mysql_cnf_file: "
+            "Cannot open %s for writing.\n%s\n",
+            new_ini_filepath,strerror(errno));
+    g_free(new_ini_filepath);
+	  g_free(bak_filepath);
+
+    return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
+  }
+
+  error= myx_update_mysql_cnf_filef(desc, ini_filepath, new_file, group);
+  
+  if (ferror(new_file))
+  {
+	  fprintf(stderr,
+			"Error in function myx_update_mysql_cnf_file: "
+			"can't write to %s.\n%s\n",
+			new_ini_filepath,strerror(ferror(new_file)));
+  }
+
+  fclose(new_file);
+  
+  if (error == MYX_ADMIN_NO_ERROR)
+  {
+	  // backup and commit 
+
+    if (file_exists(bak_filepath))
+      remove(bak_filepath);
+	
+	  if (file_exists(ini_filepath) && rename(ini_filepath, bak_filepath))
+	  {
+	    fprintf(stderr,
+			    "Error in function myx_update_mysql_cnf_file: "
+			    "can't rename file \"%s\" to \"%s\".\n%s\n",
+			    ini_filepath,bak_filepath,strerror(errno));
+	    error= MYX_ADMIN_ERROR_CANT_OPEN_FILE;
+	    goto error;
+	  }
+
+	  if (rename(new_ini_filepath, ini_filepath))
+	  {
+	    fprintf(stderr,
+			    "Error in function myx_update_mysql_cnf_file: "
+			    "can't rename file \"%s\" to \"%s\".\n%s\n",
+			    new_ini_filepath, ini_filepath, strerror(errno));
+	    error= MYX_ADMIN_ERROR_CANT_OPEN_FILE;
+	    goto error;
+	  }
+
+	  if (file_exists(bak_filepath) && remove(bak_filepath))
+	  {
+	    fprintf(stderr,
+			    "Error in function myx_update_mysql_cnf_file: "
+			    "can't remove file \"%s\".\n%s\n",
+			    bak_filepath,strerror(errno));
+	    error= MYX_ADMIN_ERROR_CANT_OPEN_FILE;
+	    goto error;
+	  }
+  }
+  
+error:
+  g_free(new_ini_filepath);
+  
+  return error;
+}
+
+
+MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_filef(MYX_GUI_DESCRIPTION *desc,
+											   const char *ini_filepath,
+											   FILE *new_file,
+											   const char *group)
 {
   FILE *ini_file;
   char buffer[MAX_LINE_LENGTH];
@@ -429,8 +511,6 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
   int our_section_has_started= 0;
   MYX_ADMIN_LIB_ERROR error_code= MYX_ADMIN_NO_ERROR;
   int section_needs_to_be_created= 1;
-  char * new_ini_filepath, * file_postfix;
-  size_t ini_filepath_len, new_ini_filepath_len;
 
   line_array= g_ptr_array_new();
   processed_widgets= g_ptr_array_new(); 
@@ -494,26 +574,6 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
     fclose(ini_file);
   }
 
-  ini_filepath_len= strlen(ini_filepath);
-  new_ini_filepath_len= ini_filepath_len + sizeof(".new");
-  new_ini_filepath= g_malloc((gulong)new_ini_filepath_len);
-  strncpy(new_ini_filepath,ini_filepath,ini_filepath_len);
-  file_postfix= new_ini_filepath+ini_filepath_len;
-  strcpy(file_postfix,".new");
-
-  if ( !(ini_file= myx_fopen(new_ini_filepath, "w")) )
-  {
-    fprintf(stderr,
-            "Error in function myx_update_mysql_cnf_file: "
-            "Cannot open %s for writing.\n%s\n",
-            new_ini_filepath,strerror(errno));
-    g_free(new_ini_filepath);
-    g_ptr_array_free(line_array,0);
-    g_ptr_array_free(processed_widgets,0);
-    fclose(ini_file);
-    return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
-  }
-
   /* we go through the list and decide for each line if it is still valid */
   for (i= 0; i< line_array->len; i++)
   {
@@ -526,7 +586,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
     {
       our_section_has_started= 1;
       section_needs_to_be_created= 0;
-      fprintf(ini_file, "%s",fl->line);
+      fprintf(new_file, "%s",fl->line);
     }
     else if (fl->type == ILT_NAME_VALUE_PAIR || fl->type == ILT_NAME_ONLY)
     {
@@ -535,7 +595,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
       if (!widget)
       {
         /* well if we don't know it we'll comment it out */
-        fprintf(ini_file, "#(Unknown option:)%s", fl->line);
+        fprintf(new_file, "#(Unknown option:)%s", fl->line);
         continue;
       }
 
@@ -543,7 +603,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
       { /* yes not nice maybe but we are always going to
         * comment them out, and write them together in
         * the end */
-        fprintf(ini_file, "#%s", fl->line);
+        fprintf(new_file, "#%s", fl->line);
         continue;
       }
 
@@ -555,17 +615,17 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
         {
           if (widget->value && widget->active)
           {
-            safe_fprintf_TEXTEDIT(ini_file,widget);
+            safe_fprintf_TEXTEDIT(new_file,widget);
           }
           else if (widget->active)
           {
             /* yes comments get lost by this, but it could 
               be that the user is using a deprecated alias */
-            fprintf(ini_file,"%s\n", widget->id);
+            fprintf(new_file,"%s\n", widget->id);
           }
           else
           {
-            fprintf(ini_file,"#%s", fl->line);
+            fprintf(new_file,"#%s", fl->line);
           }
         }
         else assert(0);
@@ -574,7 +634,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
       {
         if (!widget->active)
         {
-          fprintf(ini_file, "#%s", fl->line);
+          fprintf(new_file, "#%s", fl->line);
           continue;
         }
         if (widget->widget_type == MYX_CHECKBOX)
@@ -583,7 +643,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
 
           if (!widget->checkbox->is_boolean)
           {
-            fprintf(ini_file,
+            fprintf(new_file,
                     strcasecmp(widget->value, widget->default_value)
                     ? "%s" : "#%s", fl->line);
           }
@@ -596,17 +656,17 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
                             : NAME_ONLY_OPTION_ENABLED;
             if (!strcasecmp(w_val, widget->default_value) )
             {
-              fprintf(ini_file, "#%s", fl->line);
+              fprintf(new_file, "#%s", fl->line);
             }
             else
             {
               if ( !strcasecmp(w_val, NAME_ONLY_OPTION_ENABLED))
               {
-                fprintf(ini_file, "%s\n", widget->id);
+                fprintf(new_file, "%s\n", widget->id);
               }
               else if (!strcasecmp(w_val, NAME_ONLY_OPTION_DISABLED))
               {
-                fprintf(ini_file, "skip-%s\n", widget->id);
+                fprintf(new_file, "skip-%s\n", widget->id);
               }
               else assert(0);
             }
@@ -617,7 +677,7 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
           /* it is no checkbox */
           if ( !widget->value)
           {
-            fprintf(ini_file,"%s\n", widget->id);
+            fprintf(new_file,"%s\n", widget->id);
           }
           else if (strcasecmp(widget->value, fl->value))
           {
@@ -625,15 +685,15 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
                                                    line will get lost by this*/
             if (widget->widget_type == MYX_TEXTEDIT)
             { 
-              safe_fprintf_TEXTEDIT(ini_file,widget); 
+              safe_fprintf_TEXTEDIT(new_file,widget); 
             }
             else
             {
-              fprintf(ini_file,"%s=%s\n", widget->id, widget->value);
+              fprintf(new_file,"%s=%s\n", widget->id, widget->value);
             }
           }
           else
-            fprintf(ini_file, "%s",fl->line);
+            fprintf(new_file, "%s",fl->line);
         }
       }
     }
@@ -642,14 +702,14 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
     {
       our_section_has_started = 0;
 
-      /*before we leave our section we have to write all missing values*/
-      write_gui_desc_values_to_ini_file(desc, processed_widgets, ini_file);
+      /*before we leave our section we have to write all missning values*/
+      write_gui_desc_values_to_ini_file(desc, processed_widgets, new_file);
 
       /*let the new section finally start*/
-      fprintf(ini_file, "\n\n%s", fl->line);
+      fprintf(new_file, "\n\n%s", fl->line);
     }
     else
-      fprintf(ini_file, "%s",fl->line);
+      fprintf(new_file, "%s",fl->line);
 
     free_ini_file_line(fl);
   }
@@ -657,60 +717,19 @@ MYX_ADMIN_LIB_ERROR myx_update_mysql_cnf_file(MYX_GUI_DESCRIPTION *desc,
   if (section_needs_to_be_created)
   {
     assert(processed_widgets->len == 0);
-    fprintf(ini_file, "[%s]\n", group);
-    write_gui_desc_values_to_ini_file(desc, processed_widgets, ini_file);
+    fprintf(new_file, "[%s]\n", group);
+    write_gui_desc_values_to_ini_file(desc, processed_widgets, new_file);
   }
   else if (our_section_has_started) /* our section is the last in the file */
   {
-    write_gui_desc_values_to_ini_file(desc, processed_widgets, ini_file);
+    write_gui_desc_values_to_ini_file(desc, processed_widgets, new_file);
   }
 
   g_ptr_array_free(line_array,0);
   g_ptr_array_free(processed_widgets,0);
 
-  if (ferror(ini_file))
-  {
-    fprintf(stderr,
-            "Error in function myx_update_mysql_cnf_file: "
-            "can't write to %s.\n%s\n",
-            new_ini_filepath,strerror(errno));
-    fclose(ini_file);
-    g_free(new_ini_filepath);
+  if (ferror(new_file))
     return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
-  }
-
-  fclose(ini_file);
-
-  strcpy(file_postfix,".old");
-  if (file_exists(ini_filepath) && rename(ini_filepath,new_ini_filepath))
-  {
-    fprintf(stderr,
-            "Error in function myx_update_mysql_cnf_file: "
-            "can't rename file \"%s\" to \"%s\".\n%s\n",
-            ini_filepath,new_ini_filepath,strerror(errno));
-    g_free(new_ini_filepath);
-    return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
-  }
-  strcpy(file_postfix,".new");
-  if (rename(new_ini_filepath,ini_filepath))
-  {
-    fprintf(stderr,
-            "Error in function myx_update_mysql_cnf_file: "
-            "can't rename file \"%s\" to \"%s\".\n%s\n",
-            new_ini_filepath,ini_filepath,strerror(errno));
-    g_free(new_ini_filepath);
-    return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
-  }
-  strcpy(file_postfix,".old");
-  if (file_exists(new_ini_filepath) && remove(new_ini_filepath))
-  {
-    fprintf(stderr,
-            "Error in function myx_update_mysql_cnf_file: "
-            "can't remove file \"%s\".\n%s\n",
-            new_ini_filepath,strerror(errno));
-    g_free(new_ini_filepath);
-    return MYX_ADMIN_ERROR_CANT_OPEN_FILE;
-  }
 
   return 0;
 }
