@@ -83,7 +83,7 @@ char * q_create_user_info_base=
   " TYPE=MYISAM"
   " COMMENT='Stores additional user information'";
 
-char * q_create_user_info_4_1=
+char * q_create_user_info_utf8=
   "CREATE TABLE mysql." USER_INFO_TABLENAME " ("
   " User VARCHAR(16) NOT NULL,"
   " Full_name VARCHAR(60),"
@@ -98,6 +98,21 @@ char * q_create_user_info_4_1=
   " COMMENT='Stores additional user information'"
   " CHARACTER SET UTF8";
 
+char * q_create_user_info_utf8_bin=
+  "CREATE TABLE mysql." USER_INFO_TABLENAME " ("
+  " User VARCHAR(16) NOT NULL,"
+  " Full_name VARCHAR(60),"
+  " Description VARCHAR(255),"
+  " Email VARCHAR(80),"
+  " Contact_information TEXT,"
+  " Icon BLOB,"
+  " PRIMARY KEY(User),"
+  " INDEX user_info_Full_name(Full_name)"
+  ")"
+  " TYPE=MYISAM"
+  " COMMENT='Stores additional user information'"
+  " CHARACTER SET UTF8 COLLATE utf8_bin";
+
 int myx_check_mysql_user_info_table(MYSQL *mysql)
 {
   MYSQL_RES* res;
@@ -117,9 +132,20 @@ int myx_check_mysql_user_info_table(MYSQL *mysql)
   }
   else //If the USER_INFO_TABLE does not exist, create it
   {
+    const char *query;
+
     //Create table
-    const char *query= mysql_version_is_later_or_equal_than(mysql,4,1)
-                       ? q_create_user_info_base : q_create_user_info_4_1;
+    if (mysql_full_version_is_later_or_equal_than(mysql,5,0,3))
+      query= q_create_user_info_utf8_bin;
+    else if (mysql_version_is_later_or_equal_than(mysql,5,0))
+      query= q_create_user_info_utf8;
+    else if (mysql_full_version_is_later_or_equal_than(mysql,4,1,11))
+      query= q_create_user_info_utf8_bin;
+    else if (mysql_version_is_later_or_equal_than(mysql,4,1))
+      query= q_create_user_info_utf8;
+    else
+      query= q_create_user_info_base;
+
     mysql_free_result(res);
     result= myx_mysql_query(mysql,query) ? 0 : 1;
   }
@@ -926,14 +952,26 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
   MYSQL_RES* res;
   unsigned long cur_name_len, name_len= (unsigned long)strlen(user->user_name);
   int cur_user_has_been_set= 0;
+  int utf8_bin= 0;
+  
+  if (mysql_full_version_is_later_or_equal_than(mysql,4,1,11)
+      && (!mysql_version_is_later_or_equal_than(mysql,5,0)
+          || mysql_full_version_is_later_or_equal_than(mysql,5,0,3)))
+    utf8_bin= 1;
+
+  if (utf8_bin)
+  {
+    if (myx_mysql_query(mysql, "SET collation_connection = utf8_bin"))
+      goto error;
+  }
 
   if (myx_set_local_variable(mysql,"new_user",user->user_name,name_len))
-    return -1;
+    goto error;
 
   if (!new_user) // Update existing User
   {
     if (!(current_user= myx_get_user(mysql, previous_user_name)))
-      return -1;
+      goto error;
     cur_name_len= (unsigned long)strlen(current_user->user_name);
 
     // check if name has changed
@@ -952,7 +990,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
           myx_mysql_query(mysql,"UPDATE mysql.columns_priv SET User=@new_user"
                                 " WHERE User=@cur_user"))
       {
-        return -1;
+        goto error;
       }
       cur_user_has_been_set= 1;
     }
@@ -980,7 +1018,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         if (myx_mysql_real_query(mysql, sqlcmd,
                                  (unsigned int)(sqlcmd_end-sqlcmd)))
         {
-          return -1;
+          goto error;
         }
       }
     }
@@ -1007,7 +1045,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         if (myx_mysql_real_query(mysql, sqlcmd,
                                  (unsigned int)(sqlcmd_end-sqlcmd)))
         {
-          return -1;
+          goto error;
         }
       }
     }
@@ -1040,13 +1078,13 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       {
         g_message("Error executing query %s:\n%s",sqlcmd,
                   myx_mysql_error(mysql));
-        return -1;
+        goto error;
       }
 
       if (replace_user_info(mysql,user,name_len,
                             user_info_replace_parts[u_variant]))
       {
-        return -1;
+        goto error;
       }
     }
 
@@ -1062,7 +1100,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       if (myx_mysql_real_query(mysql, sqlcmd,
                                (unsigned int)(sqlcmd_end-sqlcmd))) 
       {
-        return -1; 
+        goto error;
       }
     }
   }
@@ -1085,7 +1123,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       if (myx_mysql_real_query(mysql, sqlcmd,
                                (unsigned int)(sqlcmd_end-sqlcmd)))
       {
-        return -1;
+        goto error;
       }
     }
 
@@ -1096,7 +1134,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       if (replace_user_info(mysql,user,name_len,
                             user_info_insert_parts))
       {
-        return -1;
+        goto error;
       }
     }
   }
@@ -1106,15 +1144,20 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
   if (user->user_object_privileges_num>0)
   {
     //Delete previous settings from mysql.db table
-    if (myx_mysql_query(mysql, "DELETE FROM mysql.db WHERE User=@new_user") ||
-        myx_mysql_query(mysql,
-                        "DELETE FROM mysql.tables_priv WHERE User=@new_user")||
-        myx_mysql_query(mysql,
-                        "DELETE FROM mysql.columns_priv WHERE User=@new_user"))
+    if (myx_mysql_query(mysql, "DELETE FROM mysql.db WHERE User=@new_user"))
     {
-      return -1;
+      char *error= myx_mysql_error(mysql);
+      int error_num= myx_mysql_errno(mysql);
+
+      goto error;
     }
 
+    if (myx_mysql_query(mysql, "DELETE FROM mysql.tables_priv WHERE User=@new_user"))
+      goto error;
+
+    if (myx_mysql_query(mysql, "DELETE FROM mysql.columns_priv WHERE User=@new_user"))
+      goto error;
+
     long_sqlcmd= g_malloc0(sizeof(char)*700);
 
     for(i= 0; i<user->user_object_privileges_num; i++)
@@ -1147,7 +1190,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                                  (unsigned int)(sqlcmd_end-long_sqlcmd)))
         {
           g_free(long_sqlcmd);
-          return -1; 
+          goto error;
         }
       }
       else if (dot_count==0) // Schema Privileges
@@ -1184,7 +1227,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                                  (unsigned int)(sqlcmd_end-long_sqlcmd)))
         {
           g_free(long_sqlcmd);
-          return -1; 
+          goto error;
         }
       }
       else // Schema Table privileges
@@ -1278,7 +1321,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
                                  (unsigned int)(sqlcmd_end-long_sqlcmd)))
         {
           g_free(long_sqlcmd);
-          return -1; 
+          goto error;
         }
       }
     }
@@ -1288,9 +1331,23 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
 
   //Flush Privileges
   if (myx_mysql_query(mysql, "FLUSH PRIVILEGES"))
-    return -1;
+    goto error;
 
+  if (utf8_bin)
+  {
+    myx_mysql_query(mysql, "SET NAMES utf8");
+    myx_mysql_query(mysql, "SET collation_connection = utf8_general_ci");
+  }
   return 0;
+
+error:
+  if (utf8_bin)
+  {
+    myx_mysql_query(mysql, "SET NAMES utf8");
+    myx_mysql_query(mysql, "SET collation_connection = utf8_general_ci");
+  }
+
+  return -1;
 }
 
 MYX_USER_OBJECT_PRIVILEGES * myx_get_privilege_struct(MYSQL *mysql, 
