@@ -203,6 +203,9 @@ bool MABackupPanel::init()
   _column_icon= PIXCACHE->load("16x16_Field.png");
 
   _schema_browser= new MGTableBrowserList("", MGTableBrowserList::Schema);
+  
+  _schema_browser->set_name("schema_list");
+  
   ((Gtk::Container*)get_widget("schemata_frame"))->add(*Gtk::manage(_schema_browser));
   _schema_browser->signal_selected().connect(SigC::slot(*this,&MABackupPanel::schema_selected));
  // _schema_browser->set_icons(_schema_icon, _sys_schema_icon, _table_icon, _column_icon);
@@ -249,6 +252,7 @@ bool MABackupPanel::init()
   ((Gtk::RadioButton*)get_widget("lockall_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
   ((Gtk::RadioButton*)get_widget("singletrans_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
   ((Gtk::RadioButton*)get_widget("normal_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
+  ((Gtk::RadioButton*)get_widget("binlogpos_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
 
   get_toggle("backup_all_check")->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
   
@@ -639,6 +643,8 @@ void MABackupPanel::show_profile(MYX_BACKUP_PROFILE *profile)
     get_toggle("singletrans_radio")->set_active(true);
   else if (profile && profile->options & MYX_B_LOCK_ALL_TABLES)
     get_toggle("lockall_radio")->set_active(true);
+  else if (profile && profile->options & MYX_B_POINT_IN_TIME_BACKUP)
+    get_toggle("binlogpos_radio")->set_active(true);
   else
     get_toggle("normal_radio")->set_active(true);
 
@@ -791,6 +797,8 @@ MYX_BACKUP_PROFILE *MABackupPanel::gather_profile()
     flags |= MYX_B_LOCK_ALL_TABLES; 
   else if (get_toggle("singletrans_radio")->get_active())
     flags |= MYX_B_SINGLE_TRANSACTION;
+  else if (get_toggle("binlogpos_radio")->get_active())
+    flags |= MYX_B_POINT_IN_TIME_BACKUP;
   
   if (get_toggle("backup_all_check"))
     flags |= MYX_B_COMPLETE_SCHEMATAS;
@@ -1018,12 +1026,27 @@ bool MABackupPanel::update_backup_progress(void *data)
   {
     _progress_label->set_text(p->table);
 
-    _row_progress->set_fraction((double)p->num_rows/p->num_rows_total);
-    _row_progress->set_text(ufmt(_("row %i of %i"), p->num_rows, p->num_rows_total));
-  
-    _table_progress->set_fraction((double)p->num_tables/p->num_tables_total);
-    _table_progress->set_text(ufmt(_("table %i of %i"), p->num_tables, p->num_tables_total));
-    
+    if (p->num_rows_total > 0)
+    {
+      _row_progress->set_fraction((double)p->num_rows/p->num_rows_total);
+      _row_progress->set_text(ufmt(_("row %i of %i"), p->num_rows, p->num_rows_total));
+    }
+    else
+    {
+      _row_progress->set_fraction(0.0);
+      _row_progress->set_text(ufmt(_("row %i of %i"), 0, 0));
+    }
+
+    if (p->num_tables_total > 0)
+    {
+      _table_progress->set_fraction((double)p->num_tables/p->num_tables_total);
+      _table_progress->set_text(ufmt(_("table %i of %i"), p->num_tables, p->num_tables_total));
+    }
+    else
+    {
+      _table_progress->set_fraction(0.0);
+      _table_progress->set_text(ufmt(_("table %i of %i"), 0, 0));
+    }
     p->ready= false;
   }
 
