@@ -255,7 +255,7 @@ class TimeGraph : public MAHealthPanel::BaseGraph {
         
         _graph->add_value(value, time(NULL));
         if (_meter)
-          _meter->set_value((int)((value-_min)*100/_max));
+          _meter->set_value((unsigned int)((value-_min)*100/_max));
         
         double max, min, avg;
         _graph->get_stats(min, max, avg);
