@@ -666,7 +666,7 @@ void MARestorePanel::perform_restore()
   arg.pdata.ready= false;
   arg.target_catalog= ""; //XXX
   arg.target_schema= get_entry("targetschema_entry")->get_text();
-  if (arg.target_schema[0]=='*') arg.target_schema="";
+  if (!arg.target_schema.empty() && arg.target_schema[0]=='*') arg.target_schema="";
   arg.options= 0;
   if (get_toggle("option1_check")->get_active())
     arg.options |= MYX_RBS_FORCE;
