@@ -2035,33 +2035,39 @@ void MAUserAdministrationPanel::user_selected(MGBrowserList *sender,
   {
     bool found= false;
     MYX_USER *user= _data->get_user(username);
-    for (unsigned int i= 0; i < user->hosts_num; i++)
-    {
-      if (host.compare((char*)user->hosts[i])==0)
-      {
-        found= true;
-        break;
-      }
-    }
-    if (!found)
-    {
-      // selected host doesn't exist (there's no % host for this user)
-      host= "";
-    }
-    show_user(username, host);
-
-    // fill list of hosts for this user
-    if (!is_host) 
-    {
-      Gtk::TreeModel::Row row= *node;
-      // if didnt refresh yet
-      if (!row[_user_browser->_columns._populated])
-      {
-        _user_browser->populate_node(node);
-      }
-    }
     
-    _user_browser->expand_node(node);
+    if (user)
+    {
+      for (unsigned int i= 0; i < user->hosts_num; i++)
+      {
+        if (host.compare((char*)user->hosts[i])==0)
+        {
+          found= true;
+          break;
+        }
+      }
+      if (!found)
+      {
+        // selected host doesn't exist (there's no % host for this user)
+        host= "";
+      }
+      show_user(username, host);
+      
+      // fill list of hosts for this user
+      if (!is_host) 
+      {
+        Gtk::TreeModel::Row row= *node;
+        // if didnt refresh yet
+        if (!row[_user_browser->_columns._populated])
+        {
+          _user_browser->populate_node(node);
+        }
+      }
+      
+      _user_browser->expand_node(node);
+    }
+    else
+      unshow_user();
   }
   else
     unshow_user();
