@@ -1,14 +1,14 @@
 #import "MABackup.h"
 #import "maUtils.h"
-#import "NSString_extras.h"
+#import <MySQLToolsCommon/NSString_extras.h>
 #include "myx_admin_public_interface.h"
-#import "MAPreferences.h"
-#import "MTextImageCell.h"
-#import "MButtonImageCell.h"
-#import "MSchemaDatasource.h"
-#import "MMySQLDispatcher.h"
-#import "MTreeDataSource.h"
-#include "myxutil.h"
+
+#import <MySQLToolsCommon/MTextImageCell.h>
+#import <MySQLToolsCommon/MButtonImageCell.h>
+#import <MySQLToolsCommon/MSchemaDatasource.h>
+#import <MySQLToolsCommon/MMySQLDispatcher.h>
+#import <MySQLToolsCommon/MTreeDataSource.h>
+#include <MySQLToolsCommon/myxutil.h>
 //XXX make a protocol with the exported stuff from MAdministrator and include it here
 extern NSString *MASchemaDataSourceChanged;
 
@@ -135,7 +135,10 @@ extern NSString *MASchemaDataSourceChanged;
   {
     id obj= [_children objectAtIndex:i];
     if ([[obj repr] isEqualToString:table])
+	{
       [obj setSelected:YES];
+	  break;
+	}
   }  
 }
 
@@ -243,7 +246,8 @@ extern NSString *MASchemaDataSourceChanged;
 
 - (void)refreshProfiles
 {
-  NSDirectoryEnumerator *en= [[NSFileManager defaultManager] enumeratorAtPath:[[MAPreferences preferences] pathForBackupProfiles]];
+  NSDirectoryEnumerator *en= 
+    [[NSFileManager defaultManager] enumeratorAtPath:[[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"]];
   NSString *file;
   [_profiles removeAllObjects];
   while ((file= [en nextObject]))
@@ -270,12 +274,23 @@ static struct {
 - (void)openProfile:(NSString*)name
 {
   MYX_ADMIN_LIB_ERROR error;
+  NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
   
   if (_profile)
     myx_free_profile(_profile);
+  _profile= NULL;
   
+  if (!name)
+  {
+    [nameText setStringValue:@""];
+    [_selectionDS setRoot:nil];
+    [contentOutline reloadData];
+    
+    return;
+  }
+    
   _profile= myx_load_profile([[name stringByAppendingString:@".mbp"] UTF8String],
-                             [[[MAPreferences preferences] pathForBackupProfiles] UTF8String],
+                             [[defaults stringForKey:@"BackupProfileDirectory"] UTF8String],
                              &error);
   if (!_profile)
   {
@@ -290,7 +305,7 @@ static struct {
     int i;
     unsigned int s;
     MTreeItem *root;
-    
+
     [nameText setStringValue:[NSString stringWithUTF8String:_profile->profile_name]];
     
     if (_profile->options & MYX_B_LOCK_ALL_TABLES)
@@ -308,7 +323,7 @@ static struct {
       btn= [optionMatrix cellWithTag:options[i].tag];
       [btn setState:(_profile->options & options[i].option)?NSOnState:NSOffState];
     }
-    
+
     root= [MTreeItem itemWithTag:'R' repr:nil];
     [_selectionDS setRoot:root];
 
@@ -323,7 +338,8 @@ static struct {
       cat= [[schemaOutline dataSource] findItem:[NSString stringWithUTF8String: table->catalog]];
       if (cat)
       {
-        sch= [cat findChild:[NSString stringWithUTF8String: table->schema]];
+        sch= [[schemaOutline dataSource] findChild:[NSString stringWithUTF8String: table->schema]
+											ofItem:cat];
         if (!sch)
         {
           NSLog(@"Invalid schema '%s.%s' in backup profile", table->catalog, table->schema);
@@ -381,7 +397,7 @@ static struct {
   _profile->backup_content= [self getContentSelection];
   
   error= myx_save_profile([[NSString stringWithFormat:@"%s.mbp",_profile->profile_name] UTF8String],
-                          [[[MAPreferences preferences] pathForBackupProfiles] UTF8String],
+                          [[[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"] UTF8String],
                           _profile);
   
   if (error != MYX_ADMIN_NO_ERROR)
@@ -395,7 +411,7 @@ static struct {
   [schemaOutline reloadData];
 }
 
-
+/*
 - (void)setItem:(id)item status:(MYX_SCHEMA_TABLE_STATUS*)status
 {
   unsigned int i;
@@ -416,8 +432,10 @@ static struct {
       }
     }
   }
+
   [contentOutline reloadData];
 }
+*/
 
 - (MYX_BACKUP_CONTENT*)getContentSelection
 {
@@ -474,7 +492,7 @@ static struct {
   if (row >= 0)
   {
     NSString *path= [NSString stringWithFormat:@"%@%@",
-      [[MAPreferences preferences] pathForBackupProfiles],
+      [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"],
       [_profiles objectAtIndex:row]];
     
     [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];
@@ -492,6 +510,7 @@ static struct {
   NSArray *schemas= [[_selectionDS root] children];
   int i,c= [schemas count];
   MABackupSelectionItem *selItem;
+  MYX_SCHEMA_TABLE_STATUS *status;
   
   for (i= 0; i < c; i++)
   {
@@ -501,13 +520,36 @@ static struct {
   }
 
   selItem= [[MABackupSelectionItem alloc] initWithSchemaItem: item];
-  [[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_table_status
+
+  status= [[_owner dispatcher] performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_table_status
                               argument:[item schema]->catalog_name
                               argument:[item schema]->schema_name
-                      finishedSelector:@selector(setItem:status:)
-                              argument:selItem
-                                target:self];
-
+						 waitForWindow:[_owner window]
+							   message:@"Retrieving table data..."];
+                      //finishedSelector:@selector(setItem:status:)
+                      //        argument:selItem
+                      //          target:self];
+  if (status)
+  {
+	[selItem resetSchemaStatus:status];
+  
+	if (_profile)
+	{
+	  // go through the current profile and select tables that were previously selected
+	  for (i= 0; i < (int)_profile->backup_content->tables_num; i++)
+	  {
+		MYX_BACKUP_TABLE *table= _profile->backup_content->tables+i;
+		if (strcmp(selItem->_catalog, table->catalog)==0
+			&& strcmp(selItem->_schema, table->schema)==0)
+		{
+		  [selItem selectTable:[NSString stringWithUTF8String:table->table]];
+		}
+	  }
+	}
+  
+	[contentOutline reloadData];
+  }
+  
   [[_selectionDS root] addChild:selItem];
   [selItem autorelease];
   
@@ -656,14 +698,16 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 - (IBAction)addProfile:(id)sender
 {
   NSFileManager *fman= [NSFileManager defaultManager];
-  NSString *prefix= [[MAPreferences preferences] pathForBackupProfiles];
+  NSString *prefix= [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"];
   NSString *name= _(@"New Profile");
   unsigned int i;
   MYX_BACKUP_PROFILE profile;
   MYX_ADMIN_LIB_ERROR err;
   
+  [profileTable deselectAll:nil];
+  
   i= 1;
-  while ([fman fileExistsAtPath:[NSString stringWithFormat:@"%@%@", prefix,name]])
+  while ([fman fileExistsAtPath:[NSString stringWithFormat:@"%@/%@.mbp", prefix,name]])
   {
     name= [NSString stringWithFormat:_(@"New Profile %i"),i++];
   }
@@ -673,7 +717,8 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   profile.options= 0;
   profile.backup_type= MYX_BT_SQL_SCRIPT;
   profile.backup_content= NULL;
-  err= myx_save_profile([[NSString stringWithFormat:@"%@.mbp",name] UTF8String], [prefix UTF8String], &profile);
+  err= myx_save_profile([[NSString stringWithFormat:@"%@.mbp",name] UTF8String], 
+                        [prefix UTF8String], &profile);
   if (err)
   {
     MARunAlertPanelWithError(@"Error", @"Could not create profile file.", err);
@@ -693,52 +738,29 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 - (IBAction)removeProfile:(id)sender
 {
   int row= [profileTable selectedRow];
-  
   if (row >= 0)
   {
-    NSString *path= [NSString stringWithFormat:@"%@%@",
-      [[MAPreferences preferences] pathForBackupProfiles],
+    NSString *path= [NSString stringWithFormat:@"%@/%@.mbp",
+      [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"],
       [_profiles objectAtIndex:row]];
-    
+
     [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];
-    [_profiles removeObjectAtIndex:row];
+    
+    if (_profile)
+      myx_free_profile(_profile);
+    _profile= NULL;
+
+    [profileTable deselectAll:nil];
+    
+    [self refreshProfiles];
     [profileTable reloadData];
   }
 }
 
-- (IBAction)selectSchema:(id)sender
-{
-  NSIndexSet *rows= [schemaOutline selectedRowIndexes];
-  NSArray *children= [[_selectionDS root] children];
-  unsigned int row;
-  BOOL flag= NO;
-  unsigned int c= [children count];
-
-  for (row= [rows firstIndex]; row <= [rows lastIndex]; row= [rows indexGreaterThanIndex:row])
-  {
-    MSchemaItem *item;
-  
-    item= [schemaOutline itemAtRow:row];
-    if (item->type == MSchemaItemType)
-    {
-      unsigned int i;
-      for (i= 0; i < c; i++)
-      {
-        if ([[children objectAtIndex:i] isEqualToItem:item])
-          break;
-      }
-      if (i == c)
-        flag= YES;
-    }
-  }
-  [contentAddButton setEnabled:flag];
-}
-
 - (IBAction)addContent:(id)sender
 {
   NSIndexSet *rows= [schemaOutline selectedRowIndexes];
   unsigned int row;
-  
   NSAssert([rows count] > 0, @"No selected schema!");
   for (row= [rows firstIndex]; row <= [rows lastIndex]; row= [rows indexGreaterThanIndex:row])
   {
@@ -749,6 +771,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
       [self addSchemaToContents:item];
     }
   }
+  [schemaOutline deselectAll:nil];
 }
 
 - (IBAction)removeContent:(id)sender
@@ -773,6 +796,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     }
   }
   [contentOutline reloadData];
+  [contentOutline deselectAll:nil];
 }
 
 - (IBAction)importProfile:(id)sender
@@ -786,7 +810,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     MYX_BACKUP_PROFILE *profile;
     MYX_ADMIN_LIB_ERROR err;
     NSString *newName;
-    NSString *profileDir= [[MAPreferences preferences] pathForBackupProfiles];
+    NSString *profileDir= [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"];
     NSString *path= [panel filename];
     int i;
     
@@ -847,9 +871,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 
 + (NSImage*)icon
 {
-  return [[[NSImage alloc] initWithContentsOfFile: 
-    [[NSBundle mainBundle] pathForResource:@"backup"
-                                    ofType:@"png"]] autorelease];
+  return [NSImage imageNamed:@"OSX-Icons_47.png"];
 }
 
 + (NSString*)label
@@ -871,9 +893,9 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     _profiles= [[NSMutableArray alloc] init];
     _selectionDS= [[MABackupDataSource alloc] init];
     [contentOutline setDataSource:_selectionDS];
-  
-    _schemaIcon= [[self loadImage:@"16x16_Database"] retain];
-    _tableIcon= [[self loadImage:@"16x16_Table"] retain];
+
+    _schemaIcon= [[NSImage imageNamed:@"16x16_Database.png"] retain];
+    _tableIcon= [[NSImage imageNamed:@"16x16_Table.png"] retain];
     
     [self refreshProfiles];
   }
@@ -891,11 +913,15 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   [super dealloc];
 }
 
-- (NSView*)sideView
+
+- (BOOL)willHide
 {
-  return profileList;
+  if (_profile)
+    [self saveProfile];
+  return YES;
 }
 
+
 - (void)awakeFromNib
 {
   MTextImageCell *tcell= [[[MTextImageCell alloc] init] autorelease];
@@ -909,13 +935,24 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   [[contentOutline tableColumnWithIdentifier:@"object"] setDataCell: bcell];  
 
   [schemaOutline setDataSource:[_owner sharedSchemaDS]];
+  MXExpandOutline(schemaOutline, NO);
   
   [nameText setDelegate:self];
   
+  [profileTable setTarget:self];
+  [profileTable setDeleteAction:@selector(removeProfile:)];
+  
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(schemaDSChanged:)
                                                name:MASchemaDataSourceChanged
                                              object:_owner];
+
+  [[[topBox viewWithTag:201] cell] setImageDimsWhenDisabled:NO];
+  [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_disabled.png"]];
+  [[topBox viewWithTag:201] setEnabled:NO];
+
+  // disable backup scheduling page for now
+  [tabView removeTabViewItem:[tabView tabViewItemAtIndex:2]];
 }
 
 
@@ -926,10 +963,18 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     
   if (_profile)
     [self saveProfile];
-  
+
   if (row >= 0)
   {
     [self openProfile: [_profiles objectAtIndex:row]];
+    [[topBox viewWithTag:201] setEnabled:YES];
+    [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_normal.png"]];
+  }
+  else
+  {
+    [self openProfile: nil];
+    [[topBox viewWithTag:201] setEnabled:NO];
+    [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_disabled.png"]];
   }
 }
 
@@ -961,6 +1006,49 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 }
 
 
+- (void)outlineViewSelectionDidChange:(NSNotification *)notification
+{
+  if ([notification object] == schemaOutline)
+  {
+    if ([schemaOutline numberOfSelectedRows] == 0 || [profileTable selectedRow] < 0)
+      [contentAddButton setEnabled:NO];
+    else
+    {
+      NSIndexSet *rows= [schemaOutline selectedRowIndexes];
+      NSArray *children= [[_selectionDS root] children];
+      unsigned int row;
+      BOOL flag= NO;
+      unsigned int c= [children count];
+    
+      for (row= [rows firstIndex]; row <= [rows lastIndex]; row= [rows indexGreaterThanIndex:row])
+      {
+        MSchemaItem *item;
+        
+        item= [schemaOutline itemAtRow:row];
+        if (item->type == MSchemaItemType)
+        {
+          unsigned int i;
+          for (i= 0; i < c; i++)
+          {
+            if ([[children objectAtIndex:i] isEqualToItem:item])
+              break;
+          }
+          if (i == c)
+            flag= YES;
+        }
+      }
+      [contentAddButton setEnabled:flag];
+    }
+  }
+  else if ([notification object] == contentOutline)
+  {
+    if ([contentOutline numberOfSelectedRows] == 0 || [profileTable selectedRow]<0)
+      [contentRemoveButton setEnabled:NO];
+    else
+      [contentRemoveButton setEnabled:YES];
+  }  
+}
+
 
 //======================== Table DataSource =====================
 
