@@ -3,9 +3,10 @@
 #import <Cocoa/Cocoa.h>
 #import "MAPanel.h"
 
+@class MFilteredSchemaDataSource;
+
 @interface MACatalogs : MAPanel
 {
-  IBOutlet NSTextField *tablesCaption;
   IBOutlet NSScrollView *tableScrollView;
   IBOutlet NSTableView *tablesTable;
   IBOutlet NSButton *tableDetailToggleButton;
@@ -14,22 +15,36 @@
   IBOutlet NSTextField *tableStatusText;
   IBOutlet NSTextField *tableStatsDataText;
   IBOutlet NSTextField *tableStatsTimeText;
-
-  IBOutlet NSTextField *indexCaption;
-  IBOutlet NSOutlineView *indexOutline;
+  IBOutlet NSPopUpButton *actionButton;
   
+  IBOutlet NSButton *addSchemaButton;
+  IBOutlet NSButton *delSchemaButton;
+
+  IBOutlet NSOutlineView *indexOutline;
+
+  IBOutlet NSSearchField *schemataSearch;
   IBOutlet NSOutlineView *schemataOutline;
-  IBOutlet NSView *sideBox;
   
   NSImage *_tableIcon;
+  NSImage *_indexIcon;
+  NSImage *_columnIcon;
+  
+  MFilteredSchemaDataSource *_schemaDS;
   
   MYX_SCHEMA_TABLE_STATUS *_tableStatus;
 }
 
 - (IBAction)toggleTableDetails:(id)sender;
-- (IBAction)openTableMaintenance:(id)sender;
+- (IBAction)checkTable:(id)sender;
+- (IBAction)repairTable:(id)sender;
+- (IBAction)optimizeTable:(id)sender;
 - (IBAction)createTable:(id)sender;
 - (IBAction)editTable:(id)sender;
+- (IBAction)dropTable:(id)sender;
 - (IBAction)refreshTables:(id)sender;
+- (IBAction)searchCatalogs:(id)sender;
+
+- (IBAction)addSchema:(id)sender;
+- (IBAction)delSchema:(id)sender;
 
 @end
