@@ -10,10 +10,11 @@
 {
     IBOutlet NSButton *ignoreErrorCheck;
     IBOutlet NSTextField *informationText;
-    IBOutlet NSButton *noCreateCheck;
+    IBOutlet NSButton *createCheck;
     IBOutlet NSComboBox *restoreTargetCombo;
     IBOutlet NSProgressIndicator *parseProgress;
     IBOutlet NSOutlineView *selectionOutline;
+    IBOutlet NSButton *restoreButton;
     
     IBOutlet NSView *accessoryView;
     IBOutlet NSPopUpButton *charsetPopUp;
@@ -21,14 +22,15 @@
     IBOutlet NSPanel *progressPanel;
     IBOutlet NSProgressIndicator *restoreProgress;
     IBOutlet NSTextField *progressText;
-    
+    IBOutlet NSTextView *warningText;
+	  
     BOOL _abortRestore;
 
     NSImage *_tableIcon;
     NSImage *_schemaIcon;
 
     MARestoreDataSource *_selectionDS;
-    
+	
     NSString *_path;
     NSString *_charset;
     MYX_BACKUP_CONTENT *_content;
