@@ -3,13 +3,17 @@
 
 #import "MAUserAdministration.h"
 #import "MUserListDataSource.h"
-#import "MSchemaDataSource.h"
-#import "MMySQLDispatcher.h"
+#import <MySQLToolsCommon/MSchemaDataSource.h>
+#import <MySQLToolsCommon/MMySQLDispatcher.h>
+#import <MySQLToolsCommon/NSView_extras.h>
 
 #import "PrivilegeDataSource.h"
 
-#import "MTextImageCell.h"
-#import "MPtrWrap.h"
+#import <MySQLToolsCommon/MOutlineView.h>
+#import <MySQLToolsCommon/MTextImageCell.h>
+#import <MySQLToolsCommon/MPtrWrap.h>
+
+#import <MySQLToolsCommon/mxUtils.h>
 
 //XXX make a protocol with the exported stuff from MAdministrator and include it here
 extern NSString *MASchemaDataSourceChanged;
@@ -33,26 +37,63 @@ extern NSString *MASchemaDataSourceChanged;
 - (BOOL)commitChanges;
 - (void)discardChanges;
 
+- (void)deleteUserPressed:(id)sender;
+
 - (void)setPage:(int)page enabled:(BOOL)enabled;
+
+- (void)setButton:(int)tag enabled:(BOOL)flag image:(NSString*)image ;
+
 @end
 
 @implementation MAUserAdministration (MAUserAdministrationPriv)
 
+- (void)deleteUserPressed:(id)sender
+{
+  if ([userOutline levelForRow:[userOutline selectedRow]] == 0)
+    [self removeUser:sender];
+  else
+    [self removeHost:sender];
+}
+
+
+- (void)setButton:(int)tag enabled:(BOOL)flag image:(NSString*)image 
+{
+  NSButton *btn= [topBox viewWithTag:tag];
+
+  if (flag)
+    [btn setImage:[NSImage imageNamed:[NSString stringWithFormat:@"%@_normal.png", image]]];
+  else
+    [btn setImage:[NSImage imageNamed:[NSString stringWithFormat:@"%@_disabled.png", image]]];
+
+  [btn setEnabled:flag];
+}
+
+
 - (void)setPage:(int)page enabled:(BOOL)enabled
 {
-  NSArray *subviews= [[[tabView tabViewItemAtIndex: page] view] subviews];
-  int c= [subviews count];
+  NSArray *subviews;
+  int c;
   int j;
   BOOL buttonsEnabled= enabled;
   
-  if (page == 2)
+  if (page == 1) // global
+    subviews= [[globalPageView view] subviews];
+  else if (page == 3) // table
+    subviews= [[tablePageView view] subviews];
+  else
+    subviews= [[[tabView tabViewItemAtIndex: page] view] subviews];
+  c= [subviews count];
+  
+  if (page == 2) // schema
   {
     if ([schemaOutline selectedRow]<0)
       buttonsEnabled= NO;
     else if ([schemaOutline levelForRow: [schemaOutline selectedRow]]<1)
       buttonsEnabled= NO;
+    else if (![self selectedUser] || ![self selectedHost])
+      buttonsEnabled= NO;
   }
-  else if (page == 3)
+  else if (page == 3) // table
   {
     if ([tableOutline selectedRow]<0)
       buttonsEnabled= NO;
@@ -77,11 +118,16 @@ extern NSString *MASchemaDataSourceChanged;
 
 - (void)schemaDSChanged: (NSNotification*)notif
 {
-  NSLog(@"schema datasource has changed");
   if ([[notif userInfo] objectForKey:@"ds"] == [_owner sharedSchemaDS])
+  {
     [schemaOutline reloadData];
+    MXExpandOutline(schemaOutline, NO);
+  }
   else
+  {
     [tableOutline reloadData];
+    MXExpandOutline(tableOutline, NO);
+  }
 }
 
 
@@ -90,10 +136,11 @@ extern NSString *MASchemaDataSourceChanged;
   NSMutableArray *array;
   MYX_USER_NAMES *names= (MYX_USER_NAMES*)result;
   unsigned int i;
-  
+
   if (!names)
   {
-    NSLog(@"could not retrieve user name list");
+    MXRunAlertPanelWithMySQLError(nil, @"Could not retrieve user list:", 
+                                  [_owner mysql]);
     return;
   }
   array= [[[NSMutableArray alloc] init] autorelease];
@@ -117,6 +164,7 @@ extern NSString *MASchemaDataSourceChanged;
                                 target:self];
 }
 
+
 - (void)userInfoArrived:(id)arg info:(void*)infoptr
 {
   MYX_USER *info= (MYX_USER*)infoptr;
@@ -125,16 +173,29 @@ extern NSString *MASchemaDataSourceChanged;
   item= [_userDS itemForUser:[NSString stringWithUTF8String:(char*)arg]];
   if (item)
   {
-    [item setUserInfo:info];
-    [userOutline reloadItem:item reloadChildren:YES];
-    
-    // if the item is selected, display it
-    if ([self selectedUser] == item)
-      [self showUser:item host: nil];
+    if (info)
+    {
+      [item setUserInfo:info];
+      [userOutline reloadItem:item reloadChildren:YES];
+      // if the item is selected, display it
+      if ([self selectedUser] == item)
+        [self showUser:item host: nil];
+    }
+    else
+    {
+      [item setUserInfo:NULL];
+      [userOutline reloadItem:item reloadChildren:YES];
+    }
   }
   g_free((char*)arg);
+  
+  if (!info)
+  {
+    NSRunAlertPanel(@"Error", @"Error retrieving user information.", nil, nil, nil);
+  }
 }
 
+
 - (BOOL)commitChanges
 {
   MUserItem *item= [self selectedUser];
@@ -187,6 +248,7 @@ extern NSString *MASchemaDataSourceChanged;
   {
     [self showMessageSheet:@"Error"
                       info:@"Could not save changes to the user."];
+    NSLog(@"Error: %s", myx_mysql_error([_owner mysql]));
   }
 
   [self setNeedsSave:NO];
@@ -333,6 +395,8 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   [userOutline reloadData];
   
   [userOutline selectRow:[userOutline rowForItem:item] byExtendingSelection:NO];
+  
+  [[topBox window] makeFirstResponder:usernameText];
 }
 
 - (IBAction)removeUser:(id)sender
@@ -344,21 +408,28 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
     {
       NSAlert *alert = [NSAlert alertWithMessageText:@"Remove Account?"
                                        defaultButton:@"Remove" alternateButton:@"Cancel" otherButton:nil        
-                           informativeTextWithFormat:@"The account will be permanently removed."];
+                           informativeTextWithFormat:@"The account '%@' will be permanently removed.", 
+                                                     [item originalName]];
       
       [alert beginSheetModalForWindow:[topBox window] modalDelegate:self 
                        didEndSelector:@selector(removeUserAlertDidEnd:returnCode:contextInfo:) contextInfo:item];
     }
+    else
+    {
+      [self removeUserAlertDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:item];
+    }
   }
 }
 
 
 - (void)removeUserAlertDidEnd:(NSAlert *)alert returnCode:(int)returnCode
-        contextInfo:(void *)contextInfo;
+        contextInfo:(void *)contextInfo
 {
   MUserItem *item= (MUserItem*)contextInfo;
   
-  if (returnCode == NSAlertFirstButtonReturn)
+  [[alert window] orderOut:self];
+  
+  if (returnCode == NSAlertDefaultReturn)
   {
     [self setNeedsSave:NO];
   
@@ -373,6 +444,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   }
 }
 
+
 - (IBAction)addHost:(id)sender
 {
   [NSApp beginSheet:addHostPanel
@@ -380,15 +452,23 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
                     modalDelegate:self
                    didEndSelector:@selector(addHostSheetDidEnd:returnCode:contextInfo:)
                       contextInfo:NULL];
-  [NSApp runModalForWindow:addHostPanel];
 }
 
 - (IBAction)removeHost:(id)sender
 {
   MUserItem *user= [self selectedUser];
   if ([self selectedHost])
+  {
+	int userRow= [userOutline selectedRow];
+
+	while (userRow >= 0 && [userOutline levelForRow:userRow]!=0)
+	  userRow--;
     [user removeHost:[self selectedHost]];
-  [self setNeedsSave:YES];
+    [self setNeedsSave:YES];
+	[userOutline selectRow:userRow
+	  byExtendingSelection:NO];
+	[userOutline reloadData];
+  }
 }
 
 - (IBAction)saveChanges:(id)sender
@@ -401,6 +481,8 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   NSOpenPanel *panel= [NSOpenPanel openPanel];
   MYX_USER *user= [[self selectedUser] userInfo];
 
+  [panel setTitle:@"Open Icon"];
+  
   if ([panel runModalForTypes:[NSArray arrayWithObject:@"png"]] == NSOKButton)
   {
     NSData *data= [NSData dataWithContentsOfFile:[panel filename]];
@@ -453,6 +535,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   [schemaAvailableTable reloadData];
   [schemaAssignedTable reloadData];
   [schemaAvailableTable deselectAll: self];  
+  
   [self setNeedsSave:YES];
 }
 
@@ -467,7 +550,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 
 - (IBAction)tableAddPrivilege:(id)sender
 {
-  grantSelectedRows(tableAvailableTable, _tablePrivDS);
+  grantSelectedRows(tableAvailableTable, [tableAvailableTable dataSource]);
   [tableAvailableTable reloadData];
   [tableAssignedTable reloadData];
   [tableAvailableTable deselectAll: self];  
@@ -476,7 +559,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 
 - (IBAction)tableRemovePrivilege:(id)sender
 {
-  revokeSelectedRows(tableAssignedTable, _tablePrivDS);
+  revokeSelectedRows(tableAssignedTable, [tableAvailableTable dataSource]);
   [tableAvailableTable reloadData];
   [tableAssignedTable reloadData];
   [tableAssignedTable deselectAll: self];
@@ -497,9 +580,7 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
 
 + (NSImage*)icon
 {
-  return [[[NSImage alloc] initWithContentsOfFile: 
-    [[NSBundle mainBundle] pathForResource:@"user_admin"
-                                    ofType:@"png"]] autorelease];
+  return [NSImage imageNamed:@"OSX-Icons_39.png"];
 }
 
 + (NSString*)label
@@ -520,6 +601,11 @@ static void revokeSelectedRows(NSTableView *table, PrivilegeDataSource *ds)
   [_schemaPrivDS release];
   [_tablePrivDS release];
   [_columnPrivDS release];
+  [_schemaDS release];
+  [_tableDS release];
+
+  [globalPageView release];
+  [tablePageView release];
   
   myx_free_user_priv(_globalPrivs);
   myx_free_user_priv(_schemaPrivs);
@@ -547,7 +633,6 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   privs->schema= myx_get_privilege_struct(mysql, "x");
   privs->table= myx_get_privilege_struct(mysql, "x.x");
   privs->column= myx_get_privilege_struct(mysql, "x.x.x");
-  
   return privs;
 }
 
@@ -565,7 +650,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   
   // setup data sources
   _userDS= [[MUserListDataSource alloc] init];
-  [_userDS setDefaultUserIcon: [self loadImage:@"16x16_User"]];
+  [_userDS setDefaultUserIcon: [NSImage imageNamed:@"16x16_User.png"]];
   [_userDS setUserInfoFetcher: self selector:@selector(requestUserInfo:)];  
   [userOutline setDataSource: _userDS];
   [[_owner dispatcher] performCallback:(void*(*)(MYSQL*))myx_get_user_names
@@ -573,81 +658,105 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
                               argument:nil
                                 target:self];
 
-  [schemaOutline setDataSource: [_owner sharedSchemaDS]];
+  _schemaDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedSchemaDS]];
+  [schemaOutline setDataSource: _schemaDS];
+  MXExpandOutline(schemaOutline, NO);
+
+  _tableDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedColumnDS]];
+  [tableOutline setDataSource: _tableDS];
+  MXExpandOutline(tableOutline, NO);
 
-  [tableOutline setDataSource: [_owner sharedColumnDS]];
-  
   {
     NSMutableArray *array;
     ALL_PRIVILEGES *privs;
     unsigned int i;
-    
     privs= [[_owner dispatcher] performCallback:(void*(*)(MYSQL*))get_all_privileges
-                                  waitForWindow:[topBox window]
+                                  waitForWindow:[_owner window]
                                         message:@"Retrieving privilege templates..."];
-
     array= [NSMutableArray array];
-    for (i= 0; i < privs->global->user_privileges_num; i++)
+    if (privs->global)
     {
-      char *end= strstr(privs->global->user_privileges[i],"_priv");
-      if (end)
-        [array addObject: [NSString stringWithCString: privs->global->user_privileges[i]
-                                               length: end-(char*)privs->global->user_privileges[i]]];
+      for (i= 0; i < privs->global->user_privileges_num; i++)
+      {
+        char *end= strstr(privs->global->user_privileges[i],"_priv");
+        if (end)
+          [array addObject: [NSString stringWithCString: privs->global->user_privileges[i]
+                                                 length: end-(char*)privs->global->user_privileges[i]]];
+      }
     }
     _globalPrivs= privs->global;
     _globalPrivDS= [[PrivilegeDataSource alloc] initWithPrivileges: array
-                                                       sourceTable: globalAvailableTable];
+                                                       sourceTable: globalAvailableTable
+                                                              type: PGlobal];
     [globalAssignedTable setDataSource: _globalPrivDS];
     [globalAssignedTable sizeToFit];
     [globalAvailableTable setDataSource: _globalPrivDS];
     [globalAvailableTable sizeToFit];
 
     array= [NSMutableArray array];
-    for (i= 0; i < privs->schema->user_privileges_num; i++)
+    if (privs->schema)
     {
-      char *end= strstr(privs->schema->user_privileges[i],"_priv");
-      if (end)
-        [array addObject: [NSString stringWithCString: privs->schema->user_privileges[i]
-                                               length: end-(char*)privs->schema->user_privileges[i]]];
+      for (i= 0; i < privs->schema->user_privileges_num; i++)
+      {
+        char *end= strstr(privs->schema->user_privileges[i],"_priv");
+        if (end)
+          [array addObject: [NSString stringWithCString: privs->schema->user_privileges[i]
+                                                 length: end-(char*)privs->schema->user_privileges[i]]];
+      }
     }
     _schemaPrivs= privs->schema;
     _schemaPrivDS= [[PrivilegeDataSource alloc] initWithPrivileges: array
-                                                       sourceTable: schemaAvailableTable];
+                                                       sourceTable: schemaAvailableTable
+                                                              type: PSchema];
     [schemaAssignedTable setDataSource: _schemaPrivDS];
     [schemaAssignedTable sizeToFit];
     [schemaAvailableTable setDataSource: _schemaPrivDS];
     [schemaAvailableTable sizeToFit];
     
     array= [NSMutableArray array];
-    for (i= 0; i < privs->table->user_privileges_num; i++)
+    if (privs->table)
     {
-      char *end= strchr(privs->table->user_privileges[i], '=');
-      if (end)
-        [array addObject: [NSString stringWithCString: privs->table->user_privileges[i]
-                                               length: end-(char*)privs->table->user_privileges[i]]];
+      for (i= 0; i < privs->table->user_privileges_num; i++)
+      {
+        char *end= strchr(privs->table->user_privileges[i], '=');
+        if (end && g_str_has_prefix(privs->table->user_privileges[i], "Table_priv_"))
+        {
+          char *begin= privs->table->user_privileges[i]+sizeof("Table_priv_")-1;
+          [array addObject: [NSString stringWithCString: begin
+                                                 length: end - begin]];
+        }
+      }
     }
     _tablePrivs= privs->table;
     _tablePrivDS= [[PrivilegeDataSource alloc] initWithPrivileges: array
-                                                       sourceTable: tableAvailableTable];
+                                                      sourceTable: tableAvailableTable
+                                                             type: PTable];
     [tableAssignedTable setDataSource: _tablePrivDS];
     [tableAvailableTable setDataSource: _tablePrivDS];
     
     array= [NSMutableArray array];
-    for (i= 0; i < privs->column->user_privileges_num; i++)
+    if (privs->column)
     {
-      char *end= strchr(privs->column->user_privileges[i], '=');
-      if (end)
-        [array addObject: [NSString stringWithCString: privs->column->user_privileges[i]
-                                               length: end-(char*)privs->column->user_privileges[i]]];
+      for (i= 0; i < privs->column->user_privileges_num; i++)
+      {
+        char *end= strchr(privs->column->user_privileges[i], '=');
+        if (end && g_str_has_prefix(privs->column->user_privileges[i], "Column_priv_"))
+        {
+          char *begin= privs->column->user_privileges[i]+sizeof("Column_priv_")-1;
+          [array addObject: [NSString stringWithCString: begin
+                                                 length: end - begin]];
+        }
+      }
     }
     _columnPrivs= privs->column;
     _columnPrivDS= [[PrivilegeDataSource alloc] initWithPrivileges: array
-                                                       sourceTable: tableAvailableTable];
+                                                       sourceTable: tableAvailableTable
+                                                              type: PColumn];
 
     g_free(privs);
   }
   
-  _privilegeIcon= [[self loadImage:@"16x16_Right"] retain];
+  _privilegeIcon= [[NSImage imageNamed:@"16x16_Right.png"] retain];
   
   [self showUser:nil host:nil];
 
@@ -655,33 +764,65 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
 }
 
 
-static void set_column_text_image(id column)
+- (void)handleDefaultsChange:(NSNotification*)notif
 {
-  if (column)
+  NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
+
+  
+  if ([defaults boolForKey:@"DisplayGlobalPrivileges"] != globalVisible)
   {
-    MTextImageCell *cell = [[[MTextImageCell alloc] init] autorelease];
-    [cell setEditable: YES];
-    [column setDataCell: cell];
+    globalVisible= !globalVisible;
+    
+    if (!globalVisible)
+      [tabView removeTabViewItem:globalPageView];
+    else
+      [tabView insertTabViewItem:globalPageView atIndex:1];
+  }
+  
+  if ([defaults boolForKey:@"DisplayTablePrivileges"] != tableVisible)
+  {
+    tableVisible= !tableVisible;
+
+    if (!tableVisible)
+      [tabView removeTabViewItem:tablePageView];  
+    else
+      [tabView insertTabViewItem:tablePageView atIndex:globalVisible?3:2];
   }
 }
 
+
 - (void)awakeFromNib
-{
-  set_column_text_image([schemaOutline tableColumnWithIdentifier:@"text"]);
-  set_column_text_image([tableOutline tableColumnWithIdentifier:@"text"]);
-  set_column_text_image([userOutline tableColumnWithIdentifier:@"text"]);
-  set_column_text_image([globalAssignedTable tableColumnWithIdentifier:@"privilege"]);
-  set_column_text_image([globalAvailableTable tableColumnWithIdentifier:@"privilege"]);
-  set_column_text_image([schemaAssignedTable tableColumnWithIdentifier:@"privilege"]);
-  set_column_text_image([schemaAvailableTable tableColumnWithIdentifier:@"privilege"]);
-  set_column_text_image([tableAssignedTable tableColumnWithIdentifier:@"privilege"]);
-  set_column_text_image([tableAvailableTable tableColumnWithIdentifier:@"privilege"]);
-}
+{  
+  [userOutline setTarget:self];
+  [userOutline setDeleteAction:@selector(deleteUserPressed:)];
+  
+  [[[topBox viewWithTag:TAG_UserAdd] cell] setImageDimsWhenDisabled:NO];
+  [[[topBox viewWithTag:TAG_UserRemove] cell] setImageDimsWhenDisabled:NO];
+  [[[topBox viewWithTag:TAG_HostAdd] cell] setImageDimsWhenDisabled:NO];
+  [[[topBox viewWithTag:TAG_HostRemove] cell] setImageDimsWhenDisabled:NO];
+  
+  [[topBox viewWithTag:300] setEnabled:NO];
+  [[topBox viewWithTag:301] setEnabled:NO];
+  [tabView setEnabledRecursive:NO];
 
+  globalPageView= [[tabView tabViewItemAtIndex:1] retain];
+  tablePageView= [[tabView tabViewItemAtIndex:3] retain];
+  
+  globalVisible= YES;
+  tableVisible= YES;
+  
+  [self handleDefaultsChange:nil];
+    
+  [[NSNotificationCenter defaultCenter] addObserver:self
+                                           selector:@selector(handleDefaultsChange:)
+                                               name:NSUserDefaultsDidChangeNotification
+                                             object:nil];
+    
 
-- (NSView*)sideView
-{
-  return sideBox;
+  [self setButton:TAG_UserAdd enabled:YES image:@"adduser"];
+  [self setButton:TAG_UserRemove enabled:NO image:@"deluser"];
+  [self setButton:TAG_HostAdd enabled:NO image:@"addhost"];
+  [self setButton:TAG_HostRemove enabled:NO image:@"delhost"];
 }
 
 
@@ -691,7 +832,14 @@ static void set_column_text_image(id column)
   NSString *tmp;
   int i;
   BOOL enabled= host!=nil;
- 
+  BOOL flag;
+  
+  flag= anUser!=nil;
+  
+  [[topBox viewWithTag:300] setEnabled:flag];
+  [[topBox viewWithTag:301] setEnabled:flag];
+  [tabView setEnabledRecursive:flag];
+  
 #define STRVALUE(v) (user ? (user->v ?[NSString stringWithUTF8String:user->v]:@"") : @"")
   [usernameText setStringValue: STRVALUE(user_name)];
   [passwordText setStringValue:@"________"]; // this is for telling the password didn't change
@@ -700,10 +848,12 @@ static void set_column_text_image(id column)
   [descriptionText setStringValue: STRVALUE(description)];
   [emailText setStringValue: STRVALUE(email)];
   [contactText setString: STRVALUE(contact_information)];
-  if (user && user->icon)
+  if ([anUser icon])
+    [iconView setImage: [anUser icon]];
+  else if (user && user->icon)
     [iconView setImage: 
-      [[[NSImage alloc] initWithData:[NSData dataWithBytesNoCopy:user->icon 
-                                                          length:user->icon_length]] autorelease]];
+      [[[NSImage alloc] initWithData:[NSData dataWithBytes:user->icon 
+                                                    length:user->icon_length]] autorelease]];
   else
     [iconView setImage: nil];
   
@@ -714,9 +864,14 @@ static void set_column_text_image(id column)
   }
   
   if (user)
-    tmp= [NSString stringWithFormat:@"%s@%@",user->user_name, host==nil?@"":host];
+  {
+    if (host)
+      tmp= [NSString stringWithFormat:@"%s@%@",user->user_name, host];
+    else
+      tmp= @"Not Applicable (no host selected)";
+  }
   else
-    tmp= @"";
+    tmp= @"No user selected";
   [userLabel1 setStringValue: tmp];
   [userLabel2 setStringValue: tmp];
   [userLabel3 setStringValue: tmp];
@@ -791,6 +946,29 @@ static void set_column_text_image(id column)
   [self setNeedsSave:YES];
 }
 
+
+- (IBAction)filterUserList:(id)sender
+{
+  [_userDS filterUsers:[sender stringValue]];
+  [userOutline reloadData];
+}
+
+
+- (IBAction)filterTableList:(id)sender
+{
+  [_tableDS performSearch:[sender stringValue]];
+  [tableOutline reloadData];
+}
+
+
+- (IBAction)filterSchemaList:(id)sender
+{
+  [_schemaDS performSearch:[sender stringValue]];
+  [schemaOutline reloadData];
+}
+
+
+
 //===================== Delegate for Outlines =======================
 
 - (BOOL)selectionShouldChangeInOutlineView:(NSOutlineView *)outlineView
@@ -837,18 +1015,23 @@ static void set_column_text_image(id column)
   if ([notification object] == userOutline)
   {
     int row;
-    
-    [userOutline setEnabled:NO];
+
     row= [userOutline selectedRow];
     if (row < 0)
-      [self showUser:NULL host:nil];
+    {
+      [self showUser:nil host:nil];
+      
+      [self setButton:TAG_UserRemove enabled:NO image:@"deluser"];
+      [self setButton:TAG_HostAdd enabled:NO image:@"addhost"];
+      [self setButton:TAG_HostRemove enabled:NO image:@"delhost"];
+    }
     else
     {
       id item= [userOutline itemAtRow: row];
       if ([item isMemberOfClass: [MUserHostItem class]]) // selected a host
       {
         MUserItem *info= [(MUserHostItem*)item user];
-        [self showUser:info host:[self selectedHost]];
+        [self showUser:info host:[self selectedHost]];        
       }
       else if ([item isMemberOfClass: [MUserItem class]]) // selected a user
       {
@@ -858,9 +1041,14 @@ static void set_column_text_image(id column)
           [self requestUserInfo:[item name]];
         }
         else
-          [self showUser:item host:nil];
+          [self showUser:item host:nil];        
       }
+      [self setButton:TAG_HostRemove enabled:YES image:@"delhost"];
+      [self setButton:TAG_UserRemove enabled:YES image:@"deluser"];
+      [self setButton:TAG_HostAdd enabled:YES image:@"addhost"];      
     }
+    [schemaOutline deselectAll:nil];
+    [tableOutline deselectAll:nil];
   }
   else if ([notification object] == schemaOutline)
   {
@@ -880,7 +1068,7 @@ static void set_column_text_image(id column)
         NSString *object= [NSString stringWithFormat:@"%s.%s",schema->catalog_name,schema->schema_name];
         privs= [user findPrivilegesForObject:object host:host];
         if (!privs)
-          privs= [user addPrivileges:_schemaPrivs forObject:object host:host];          
+          privs= [user addPrivileges:_schemaPrivs forObject:object host:host];
         [_schemaPrivDS setPrivileges:privs];
       }
       else
@@ -897,6 +1085,7 @@ static void set_column_text_image(id column)
   {
     MSchemaItem *item= nil;
     MYX_USER_OBJECT_PRIVILEGES *privs= NULL;
+    BOOL editable= NO;
 
     int row= [tableOutline selectedRow];
     if (row >= 0)
@@ -905,7 +1094,6 @@ static void set_column_text_image(id column)
     {
       MYX_SCHEMA *schema= [item schema];
       MYX_SCHEMA_TABLE *table= [item table];
-
       if (table && host && user)
       {
         NSString *object=[NSString stringWithFormat:@"%s.%s.%s",schema->catalog_name,schema->schema_name,table->table_name];
@@ -913,6 +1101,7 @@ static void set_column_text_image(id column)
         if (!privs)
           privs= [user addPrivileges:_tablePrivs forObject:object host:host];
         [_tablePrivDS setPrivileges:privs];
+        editable= YES;
       }
       else
       {
@@ -935,19 +1124,20 @@ static void set_column_text_image(id column)
         privs= [user findPrivilegesForObject:object host:host];
         if (!privs)
           privs= [user addPrivileges:_columnPrivs forObject:object host:host];
-        [_tablePrivDS setPrivileges:privs];
+        [_columnPrivDS setPrivileges:privs];
+        editable= YES;
       }
       else
       {
-        [_tablePrivDS setPrivileges:NULL];
+        [_columnPrivDS setPrivileges:NULL];
       }
-      [tableAssignedTable setDataSource:_tablePrivDS];
-      [tableAvailableTable setDataSource:_tablePrivDS];
+      [tableAssignedTable setDataSource:_columnPrivDS];
+      [tableAvailableTable setDataSource:_columnPrivDS];
       [tableAssignedTable reloadData];
       [tableAvailableTable reloadData];
     }
     // refresh enabled controls
-    [self setPage:3 enabled:YES];
+    [self setPage:3 enabled:editable];
   }
 }
 
@@ -958,8 +1148,10 @@ static void set_column_text_image(id column)
 {
   if (outlineView == schemaOutline || outlineView == tableOutline)
   {
-    if ([item respondsToSelector:@selector(icon)]) 
+    if ([item respondsToSelector:@selector(icon)])
       [cell setImage:[item icon]];
+    
+
   }
   else if (outlineView == userOutline)
   {
