@@ -10,31 +10,15 @@
 
 #import "MAXMLGUIController.h"
 #import "maUtils.h"
+#import <MySQLToolsCommon/NSView_Extras.h>
 
+#import "MADataFileView.h"
 #import "MAFileTextField.h"
 
 #ifndef MAX
 #define MAX(a,b) ((a)>(b))?(a):(b)
 #endif
 
-@interface NSView(MyExts)
-- (void)autoCenterAt:(float)y expand:(BOOL)expand;
-@end
-
-@implementation NSView(MyExts)
-- (void)autoCenterAt:(float)y expand:(BOOL)expand
-{
-  NSSize psize= [[self superview] frame].size;
-  NSRect rect= [self frame];
-  [self setFrameOrigin: NSMakePoint((psize.width-rect.size.width)/2,y)];
-  if (expand)
-    [self setAutoresizingMask:NSViewMinYMargin|NSViewWidthSizable];
-  else
-    [self setAutoresizingMask:NSViewMinYMargin|NSViewMinXMargin|NSViewMaxXMargin];
-}
-@end
-
-
 
 @interface MAXMLGUIController(Private)
 - (void)buildPage:(MYX_GUI_PAGE*)page
@@ -42,6 +26,7 @@
 - (NSView*)buildGroup:(MYX_GUI_GROUP*)group width:(float)width;
 - (void)takeValuesFromPage:(MYX_GUI_PAGE*)page;
 - (void)updateValuesInPage:(MYX_GUI_PAGE*)page;
+- (void)toggleOptionEnabled:(id)sender;
 @end
 
 static NSTextField *makeLabel(const char *text)
@@ -76,10 +61,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     {      
       if (strcmp2((char*)widget->textedit->edit_type,"innodbfilepath")==0)
       {
-        NSTextField *tf= [[[NSTextField alloc] init] autorelease];
-        [tf setFrameSize:NSMakeSize(80.0, 22.0)];
-        wid= tf;
-        //XXX TODO
+        wid= [[MADataFileView alloc] init];
       }
       else if (strcmp2((char*)widget->textedit->edit_type,"directory")==0)
       {
@@ -172,6 +154,17 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
 
 @implementation MAXMLGUIController(Private)
 
+- (void)toggleOptionEnabled:(id)sender
+{
+  id opid= [[sender cell] representedObject];
+  
+  if ([sender state] == NSOnState)
+    [[_optionCaptions objectForKey:opid] setTextColor:[NSColor blackColor]];
+  else
+    [[_optionCaptions objectForKey:opid] setTextColor:[NSColor lightGrayColor]];
+  [[_optionControls objectForKey:opid] setEnabledRecursive:[sender state]==NSOnState];
+}
+
 - (NSView*)buildGroup:(MYX_GUI_GROUP*)group width:(float)width
 {
   NSBox *box= [[[NSBox alloc] init] autorelease];
@@ -179,7 +172,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   float label_width= 0.0;
   int w;
   NSFont *font= [NSFont systemFontOfSize:[NSFont systemFontSize]];
-  NSFont *lfont= [NSFont labelFontOfSize:[NSFont labelFontSize]];
+  NSFont *lfont= [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
   
   [box setTitle:[NSString stringWithUTF8String:(char*)group->caption?:""]];
   [box setBoxType:NSBoxPrimary];
@@ -201,17 +194,20 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   for (w= group->widgets_num-1; w >= 0; w--)
   {
     NSView *widget= nil;
+    NSButton *enableBtn= nil;
     NSTextField *label= nil;
     NSTextField *descr= nil;
     NSRect rect;
     float h= 0.0;
-    float dh;
+    float dh= 0.0;
     float left_width= 0.0;
+    NSString *optionId= [NSString stringWithCString:group->widgets[w].id];
 
     if (group->widgets[w].widget_type == MYX_CHECKBOX)
     {
       widget= makeWidget(group->widgets+w);
       [box addSubview:widget];
+	  [widget setToolTip:[NSString stringWithFormat:@"Option: %s", group->widgets[w].id]];
       rect= [widget frame];
       rect.origin.x= 17.0-5.0 + 20;
       rect.origin.y= y;
@@ -223,12 +219,25 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     }
     else
     {
+      enableBtn= [[[NSButton alloc] initWithFrame:NSMakeRect(17.0-5.0,y+5.0,16,16)] autorelease];
+      [box addSubview:enableBtn];
+      [enableBtn setButtonType:NSToggleButton];
+      [enableBtn setAutoresizingMask:NSViewMaxXMargin];
+      [enableBtn setImage:_disabledImg];
+      [enableBtn setAlternateImage:_editableImg];
+      [enableBtn setImagePosition:NSImageOnly];
+      [[enableBtn cell] setBordered:NO];
+      [enableBtn setTarget:self];
+      [enableBtn setAction:@selector(toggleOptionEnabled:)];
+	  [enableBtn setToolTip:@"Indicates whether the option should be written to the configuration file.\nClick to toggle."];
+      [[enableBtn cell] setRepresentedObject:optionId];
+
       label= makeLabel((char*)group->widgets[w].caption?:"");
       [box addSubview:label];
       [label setAlignment:NSRightTextAlignment];
       rect= [label frame];
       rect.size.width= label_width+5.0;
-      rect.origin.x= 17.0-5.0;
+      rect.origin.x= 17.0-5.0+16;
       rect.origin.y= y+5.0;
       [label setAutoresizingMask:NSViewMaxXMargin];
       [label setFrame:rect];
@@ -238,10 +247,11 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
       [box addSubview:widget];
       rect= [widget frame];
       rect.size.width= [widget frame].size.width;
-      rect.origin.x= 17.0+label_width+8.0;
+      rect.origin.x= 17.0+16.0+label_width+8.0;
       rect.origin.y= y;
       [widget setAutoresizingMask:NSViewMaxXMargin|NSViewMaxYMargin];
       [widget setFrame:rect];
+	  [widget setToolTip:[NSString stringWithFormat:@"Option: %s", group->widgets[w].id]];
       h= MAX(h, rect.size.height);
       
       left_width= label_width + rect.size.width;
@@ -252,7 +262,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     [descr setFont:lfont];
     
     [[descr cell] setWraps:YES];
-    rect.origin.x= 17.0+8.0+left_width+8.0;
+    rect.origin.x= 17.0+16.0+8.0+left_width+8.0;
     rect.origin.y= y - (dh-22.0)/2.0;
     
     dh= ceil([[descr font] widthOfString:[descr stringValue]]/(width-rect.origin.x-50));
@@ -270,6 +280,11 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
       h= MAX(h, rect.size.height);
     
     // center the objects vertically according to the total height
+    if (enableBtn)
+    {
+      rect= [enableBtn frame];
+      [enableBtn setFrameOrigin:NSMakePoint(rect.origin.x, y + (h - rect.size.height)/2)];
+    }
     if (label)
     {
       rect= [label frame];
@@ -287,7 +302,11 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     }
     
     // register the control
-    [_optionControls setObject:widget forKey:[NSString stringWithCString:group->widgets[w].id]];
+    [_optionControls setObject:widget forKey:optionId];
+    if (enableBtn)
+      [_optionToggles setObject:enableBtn forKey:optionId];
+    if (label)
+      [_optionCaptions setObject:label forKey:optionId];
     
     y+= h+17.0;
   }
@@ -316,8 +335,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   [sview setHasHorizontalScroller:NO];
   [sview setHasVerticalScroller:YES];
   [sview setBorderType:NSGrooveBorder];
-  [sview autoCenterAt:17.0 expand:YES];
-  [sview setFrame:NSMakeRect(11.0, 17.0, tsize.width-11.0*2, tsize.height-17.0*2-20.0)];
+  [sview setFrame:NSMakeRect(5.0, 17.0, tsize.width-5.0*2, tsize.height-17.0*2-10.0)];
   [sview setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable|NSViewMinYMargin];  
   [sview setDocumentView:topv];
 
@@ -332,7 +350,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     NSView *groupv= [self buildGroup:page->groups+i width:width];
     float height= [groupv frame].size.height;
     [groupv setAutoresizingMask:NSViewWidthSizable];
-    [groupv setFrame:NSMakeRect(5.0, y, width-5.0*2, height)];
+    [groupv setFrame:NSMakeRect(15.0, y, width-15.0*2, height)];
     [topv addSubview:groupv];
     y+= height + 10.0;
   }
@@ -346,13 +364,15 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
       NSView *ch= [children objectAtIndex:i];
       [ch setAutoresizingMask:NSViewWidthSizable|NSViewMaxYMargin];
       yy-= [ch frame].size.height;
-      [ch setFrameOrigin:NSMakePoint(5.0, yy)];
+      [ch setFrameOrigin:NSMakePoint(15.0, yy)];
       yy-= 10.0;
     }
     [topv setFrameSize:NSMakeSize(width, svheight)];
   }
   else
+  {
     [topv setFrameSize:NSMakeSize(width, y)];
+  }
   [topv release];
   
   [topv scrollPoint:NSMakePoint(0,y)];
@@ -361,7 +381,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   textf= makeLabel(page->description);
   [pagev addSubview:textf];
   [textf setAlignment:NSCenterTextAlignment];
-  [textf setFrame:NSMakeRect(17.0,tsize.height-10.0-[textf frame].size.height,
+  [textf setFrame:NSMakeRect(17.0,tsize.height-4.0-[textf frame].size.height,
                              [pagev frame].size.width-17.0*2, [textf frame].size.height)];
   [textf setAutoresizingMask:NSViewMinYMargin|NSViewWidthSizable];
   
@@ -382,7 +402,8 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     for (w= 0; w < group->widgets_num; w++)
     {
       MYX_GUI_WIDGET *widget= group->widgets+w;
-      id view= [_optionControls objectForKey:[NSString stringWithUTF8String:widget->id]];
+      NSString *optionId= [NSString stringWithUTF8String:widget->id];
+      id view= [_optionControls objectForKey:optionId];
       char *value= widget->value;
       if (!value) value= widget->default_value;
       if (!view)
@@ -390,6 +411,10 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
         NSLog(@"view for option %s not found", widget->id);
         continue;
       }
+
+      [[_optionToggles objectForKey:optionId] setState:!widget->active?NSOnState:NSOffState];
+      [[_optionToggles objectForKey:optionId] performClick:self];
+
       switch (widget->widget_type)
       {
         case MYX_CHECKBOX:
@@ -474,12 +499,20 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     for (w= 0; w < group->widgets_num; w++)
     {
       MYX_GUI_WIDGET *widget= group->widgets+w;
-      id view= [_optionControls objectForKey:[NSString stringWithUTF8String:widget->id]];
+      NSString *optionId= [NSString stringWithUTF8String:widget->id];
+      id view= [_optionControls objectForKey:optionId];
       if (!view)
       {
         NSLog(@"view for option %s not found", widget->id);
         continue;
       }
+
+      if (widget->widget_type == MYX_CHECKBOX)
+        widget->active= 1;
+      else
+        widget->active= [[_optionToggles objectForKey:optionId] state] == NSOnState;
+      [view setEnabledRecursive:widget->active];
+      
       switch (widget->widget_type)
       {
         case MYX_CHECKBOX:
@@ -520,7 +553,7 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
           NSPopUpButton *pop= [view viewWithTag:2];
           
           NSString *str= [NSString stringWithFormat:@"%@%@", [[view viewWithTag:0] stringValue], 
-            [[pop selectedItem] title]];
+						[[pop selectedItem] title]?:@""];
           
           g_free(widget->value);
           widget->value= g_strdup([str UTF8String]);
@@ -546,8 +579,12 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
 - (void)dealloc
 {
   [_optionControls release];
+  [_optionCaptions release];
+  [_optionToggles release];
   if (_descr)
     myx_free_gui_description(_descr);
+  [_editableImg release];
+  [_disabledImg release];
   [_file release];
   [_section release];
   [super dealloc];
@@ -568,7 +605,13 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
     _file= [file retain];
     _section= [section retain];
     
+    _editableImg= [[NSImage imageNamed:@"variable_editable_mac.png"] retain];
+
+    _disabledImg= [[NSImage imageNamed:@"variable_disabled_mac.png"] retain];
+        
+    _optionToggles= [[NSMutableDictionary alloc] init];
     _optionControls= [[NSMutableDictionary alloc] init];
+    _optionCaptions= [[NSMutableDictionary alloc] init];
     _descr= myx_get_gui_description([[[NSBundle mainBundle] pathForResource:@"mysqladmin_startup_variables_description"
                                                                     ofType:@"xml"] UTF8String],
                                     version, MYX_MACOS, &err,
@@ -599,39 +642,36 @@ static NSView *makeWidget(MYX_GUI_WIDGET *widget)
   }
 }
 
-- (BOOL)save
+
+- (BOOL)saveTo:(int)fd
 {
   MYX_ADMIN_LIB_ERROR rc;
-  unsigned int i, num=0;
-  NSString *bkfile;
-  NSFileManager *fman= [NSFileManager defaultManager];
+  unsigned int i;
+  FILE *file;
   
-  // backup file
-  bkfile= [NSString stringWithFormat:@"%@.maold",_file];
-  while ([fman fileExistsAtPath:bkfile])
-  {
-    bkfile= [NSString stringWithFormat:@"%@.maold%i",_file,num++];
-  }
-  if (copy_file([_file UTF8String], [bkfile UTF8String]) < 0)
-  {
-    int e= errno;
-    NSRunAlertPanel(@"Error", @"%@\n%s", @"OK", nil, nil,
-                    [NSString stringWithFormat:@"There was an error backing up old configuration data to '%s'",[bkfile UTF8String]],
-                    strerror(e));
-    return NO;
-  }
-
   // fetch values  
   for (i= 0; i < _descr->pages_num; i++)
   {
     [self updateValuesInPage:_descr->pages+i];
   }
   
-  rc= myx_update_mysql_cnf_file(_descr, [_file UTF8String], [_section UTF8String]);
+  file= fdopen(fd, "w");
+  if (!file)
+  {
+	NSRunAlertPanel(nil, [NSString stringWithFormat:@"Error updating configuration file:%s.", strerror(errno)],
+					@"OK", nil, nil);
+	return NO;
+  }
+  
+  rc= myx_update_mysql_cnf_filef(_descr, [_file fileSystemRepresentation],
+								 file, [_section UTF8String]);
+  fclose(file);
   if (rc != MYX_ADMIN_NO_ERROR)
   {
     MARunAlertPanelWithError(@"Error", @"Error updating configuration file.", rc);
+	return NO;
   }
+
   return YES;
 }
 
