@@ -17,9 +17,19 @@
 
 - (MMySQLDispatcher*)dispatcher;
 
+- (MYSQL*)mysql;
 - (MYX_USER_CONNECTION*)serverInfo;
 
+- (int)serverStatus;
+- (BOOL)isLocal;
+
+- (NSWindow*)window;
+
+- (void)refreshSchemata;
+
 - (MSchemaDataSource*)sharedSchemaDS;
 - (MSchemaDataSource*)sharedColumnDS;
 
+- (MYX_CATALOGS*)catalogList;
+
 @end
