@@ -23,7 +23,7 @@
 
 #include "myx_public_interface.h"
 #include "myx_aux_functions.h"
-#include <myx_shared_aux_functions.h>
+#include "myx_shared_aux_functions.h"
 
 #define DBSTR(mysql, str) myx_convert_dbstr_utf8(mysql, str)
 
