@@ -19,9 +19,10 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <glib.h>
+//#include <glib.h>
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+  #include <winsock2.h>
   #include <windows.h>
 #endif
 
