@@ -205,10 +205,17 @@ static void csv_exporter_columns(MYX_TABLE_EXPORTER_INFO *self, const char *cons
     }
     else
     {
-      if (i > 0)
-        fprintf(self->file_stream, "%s%s", self->option_values[0], text[i]);
+      const char *s;
+      
+      if (text[i])
+        s= text[i];
       else
-        fprintf(self->file_stream, "%s", text[i]);
+        s= "";
+      
+      if (i > 0)
+        fprintf(self->file_stream, "%s%s", self->option_values[0], s);
+      else
+        fprintf(self->file_stream, "%s", s);
     }
   }
   fprintf(self->file_stream, "\n");
