@@ -18,6 +18,9 @@
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
+#ifdef __GNUC__
+#include <unistd.h>
+#endif
 
 #include "myx_library.h"
 #include "myx_network.h"
@@ -404,7 +407,8 @@ void myx_mysql_set_query_hooks(MYSQL *mysql, MYX_QUERY_CALLBACK pre_hook,
 int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 {
   MYSQL_REAL_CONNECT_PARAMETERS connect_options= { mysql, 0, 0};  
-
+  char *unix_socket= NULL;
+  
   /* setup advanced options */
   char **advanced_option= user_conn->advanced_options;
   char **end_advanced_option= advanced_option+user_conn->advanced_options_num;
@@ -424,23 +428,48 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
     }
   }
 
+#if !(defined(__WIN__) || defined(_WIN32) || defined(_WIN64))
+  if (!connect_options.unix_socket)
+  {
+    gchar *out= NULL, *err= NULL;
+    gint est;
+    if (g_spawn_command_line_sync("mysql_config --socket",
+                                  &out, &err, &est,
+                                  NULL) && out && est == 0)
+    {
+      char *ptr= strchr(out, '\n');
+      if (ptr)
+      {
+        *ptr= 0;
+        unix_socket= g_strdup(out);
+      }
+    }
+    g_free(out);
+    g_free(err);
+  }
+#endif
+  
   //This stmt tells the server that the parameters of
   //of mysql_real_connect are in utf8
   //mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
 
   if (!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
                           user_conn->password, user_conn->schema,
-                          user_conn->port, connect_options.unix_socket,
+                          user_conn->port, 
+                          unix_socket ? unix_socket : connect_options.unix_socket,
                           connect_options.client_flag | EXTRA_CONNECT_FLAGS))
   {
+    g_free(unix_socket);
     return -1;
   }
 
+  g_free(unix_socket);
+
   // set a high timeout value
   mysql_query(mysql, "SET SESSION interactive_timeout=1000000");
 
   // make sure we're not in ansi_quotes mode, unless asked to
-  if (mysql_version_is_later_or_equal_than(mysql,4,0))
+  if (mysql_version_is_later_or_equal_than(mysql,4,1))
   {
     if (connect_options.ansi_quotes)
       mysql_query(mysql, "SET SESSION sql_mode='ANSI_QUOTES'");
@@ -486,12 +515,9 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 *//////////////////////////////////////////////////////////////////////////////
 int myx_mysql_close(MYSQL *mysql)
 {
-  MYX_MYSQL *priv;
-  
   mysql_close(mysql);
   
-  priv= myx_mysql_get_private(mysql);
-  g_free(priv);
+  g_free(mysql);
 
   return 0;
 }
@@ -739,9 +765,6 @@ MYX_MACHINE_INFO * myx_get_server_info(MYX_USER_CONNECTION *user_conn,
   return machine_info;
 }
 
-#ifdef __GNUC__
-#include <unistd.h>
-#endif
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get information about the client machine
     @ingroup Host_information_management
@@ -801,10 +824,9 @@ int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
 }
 
 
-//#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 /*
  *----------------------------------------------------------------------
- * myx_match_string
+ * myx_match_pattern
  *
  * SYNOPSIS
  *   Match a string to a search pattern.
@@ -822,28 +844,27 @@ int myx_free_pc_info(MYX_MACHINE_INFO *machine_info)
  * NOTES
  * 
  * Supported wild characters:
- *   %  match anything, any number of chars
- *   _  match a single character
+ *   *  match anything, any number of chars
+ *   ?  match a single character
  *
  *----------------------------------------------------------------------
  */
-static gchar dummy(gchar c)
+static gunichar dummy(gunichar c)
 {
   return c;
 }
 
-int myx_match_string(const char *text, const char *search, int case_sensitive)
+int myx_match_pattern(const char *text, const char *search, int case_sensitive)
 {
-#define MANY '%'
-#define SINGLE '_'
-
 #ifdef __GNUC__
   const char *stack[strlen(text)+2];
 #else
   const char **stack= g_malloc(sizeof(char)*((unsigned int)strlen(text)+2));
 #endif
   int stackp= 0;
-  gchar (*normcase)(gchar)= case_sensitive ? dummy : g_ascii_toupper;
+  const char *tend= text+g_utf8_strlen(text, (gssize)strlen(text));
+  const char *send= search+g_utf8_strlen(text, (gssize)strlen(search));
+  gunichar (*normcase)(gunichar)= case_sensitive ? dummy : g_unichar_toupper;
 
   if (!search || !*search)
     return -1;
@@ -852,24 +873,24 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
   {
     switch (*search)
     {
-     case MANY:
-      ++search;
-      if (*search != '_')
+     case '*':
+      search= g_utf8_next_char(search);
+      if (*search != '?')
       {
-        gchar norm_search= normcase(*search);
+        gunichar norm_search= normcase(*search);
         stack[stackp++]= search;
-        while (*text && *search && normcase(*text) != norm_search)
-          ++text;
+        while (*text && *search && normcase(g_utf8_get_char_validated(text, (gssize)(tend-text))) != norm_search)
+          text= g_utf8_next_char(text);
         break;
       }
 
-     case SINGLE:
-      ++search;
-      ++text;
+     case '?':
+      search= g_utf8_next_char(search);
+      text= g_utf8_next_char(text);
       break;
       
      default:
-      if (normcase(*text) != normcase(*search))
+      if (normcase(g_utf8_get_char_validated(text, (gssize)(tend-search))) != normcase(g_utf8_get_char_validated(search, (gssize)(send-search))))
       {
         if (stackp==0)
         {
@@ -879,14 +900,14 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
         {
           // backtrack
           search= stack[--stackp];
-          while (*text && *search && normcase(*text) != normcase(*search))
-            ++text;
+          while (*text && *search && normcase(g_utf8_get_char_validated(text, (gssize)(tend-text))) != normcase(g_utf8_get_char_validated(search, (gssize)(send-search))))
+            text= g_utf8_next_char(text);
         }
       }
       else
       {
-        ++search;
-        ++text;
+        search= g_utf8_next_char(search);
+        text= g_utf8_next_char(text);
       }
       break;
     }
@@ -897,7 +918,6 @@ int myx_match_string(const char *text, const char *search, int case_sensitive)
 #endif
   return !*search;
 }
-//#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get charset name of server
@@ -1063,7 +1083,7 @@ char *myx_convert_utf8_dbstr_with_length(MYSQL *mysql, const char* str,
     //Get server charset
     res= g_convert(str, str_length,
                    iconv_char_name(myx_get_server_charset_name(mysql)),
-                   "UTF-8", &bytes_read, new_length, &error);
+                   "UTF-8", &bytes_read, (gsize*)new_length, &error);
 
     //check if it worked
     if (!error)
@@ -1113,6 +1133,91 @@ void myx_free_lib_str(char *str)
   g_free(str);
 }
 
+
+static char *escape_query(int quote, const char *cmd, va_list args)
+{
+  char *sql= g_strdup(cmd);
+  char quotestr[8];
+  char *query;
+
+  quotestr[0]= (char) quote;
+  quotestr[1]= '%';
+  quotestr[2]= 's';
+  quotestr[3]= (char) quote;
+  quotestr[4]= 0;
+
+  //sprintf((char *)quotestr, "%c%%s%c", quote, quote);
+
+  sql= str_g_replace(sql, "%s", quotestr);
+
+  query= g_strdup_vprintf(sql, args);
+  
+  g_free(sql);
+  
+  return query;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+/** @brief escapes identifiers refered in cmd with %s and given in ...
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql connection handler
+    @param cmd   text of query to execute
+    @param ...   database object names
+
+    @return the escaped sql string, has to be freed with g_free
+*//////////////////////////////////////////////////////////////////////////////
+char * myx_mysql_esc(MYSQL *mysql, const char *cmd, ...)
+{
+  va_list args;
+  char *query;
+  int quote;
+  
+  va_start(args, cmd);
+  quote= myx_get_mysql_quote_char(mysql, 0);
+
+  query= escape_query(quote, cmd, args);
+  va_end(args);
+
+  return query;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+/** @brief execute sql query using mysql connection
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql connection handler
+    @param cmd   text of query to execute
+    @param ...   database object names
+
+    @return result of mysql_query
+      (Zero if the query was successful. Non-zero if an error occurred)
+
+    The same as myx_mysql_query() but escapes defined in the cmd string with %s and
+    given in ...
+    cmd should be in UTF8, function tries to myx_convert_utf8_dbstr() it
+*//////////////////////////////////////////////////////////////////////////////
+int myx_mysql_query_esc(MYSQL *mysql, const char *cmd, ...)
+{
+  va_list args;
+  char *query;
+  int quote;
+  int res;
+  
+  va_start(args, cmd);
+  quote= myx_get_mysql_quote_char(mysql, 0);
+
+  query= escape_query(quote, cmd, args);
+  va_end(args);
+
+  res= myx_mysql_query(mysql, query);
+
+  g_free(query);
+
+  return res;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief execute sql query using mysql connection
     @ingroup bridge_to_MySQL_API
@@ -1123,7 +1228,7 @@ void myx_free_lib_str(char *str)
     @return result of mysql_query
       (Zero if the query was successful. Non-zero if an error occurred)
 
-    cmd should be in UTF8, function tryes to myx_convert_utf8_dbstr() it
+    cmd should be in UTF8, function tries to myx_convert_utf8_dbstr() it
 *//////////////////////////////////////////////////////////////////////////////
 int myx_mysql_query(MYSQL *mysql, const char *cmd)
 {
@@ -1264,7 +1369,7 @@ int myx_check_utf8_convertible(const char *str, const char *encoding)
 {
   GIConv ic= g_iconv_open(encoding, "utf-8");
   char buffer[256];
-  int in_left, out_left, count;
+  gsize in_left, out_left, count;
   char *in, *out;
   if (!ic)
   {
