@@ -337,11 +337,15 @@ struct const_name : public const_string
 
   friend std::string & operator << (std::string & dst, const const_name & src)
   {
-    if (!myx_identifier_needs_quotes(src.data()))
+    // always add quotes because otherwise we'd have to check for reserved
+    // words (like table, from, to, select etc)
+    // it's easier to just quote everything.
+/*    if (!myx_identifier_needs_quotes(src.data()))
     {
       dst << (const_string&)src;
     }
     else
+ */
     {
       dst.append(1,src.quote_symbol);
       for (const char * c= src.begin(); c!=src.end(); c++)
@@ -393,8 +397,7 @@ struct Query : public MYX_QUERY
   Query();
   ~Query();
 
-  std::string m_sql;
-  std::string m_masked_sql;
+  std::string m_sql, m_sql_qithout_comments, m_masked_sql;
 
   typedef std::vector<const char *> Type_options;
   typedef std::vector<const_string> Type_const_string_options;
@@ -683,7 +686,10 @@ MYX_Q_TABLE *Query::get_table(const char * catalog,
 ///////////////////////////////////////////////////////////////////////////////
 void Query::analyze(MYSQL *mysql, const char *sql)
 {
-  m_masked_sql= m_sql= sql;
+  m_sql_qithout_comments= sql;
+  m_sql= sql;
+  kill_comments((char*)m_sql_qithout_comments.c_str());
+  m_masked_sql= m_sql_qithout_comments.c_str();
   mask_quotas_and_brackets((char*)m_masked_sql.c_str());
   myx_get_mysql_quote_char(mysql,1);
   query_type= calc_query_type();
@@ -904,7 +910,7 @@ void query_build_clause(Query * query,
   MYX_Q_CLAUSE_TYPE clause_type;
   const const_string & start_keyword= cur_clause->start_keyword;
   const char * begin= strfindword(masked_sql.data(),start_keyword.data());
-  const char * sql= query->m_sql.c_str();
+  const char * sql= query->m_sql_qithout_comments.c_str();
   if (begin && (begin + start_keyword.length())<=masked_sql.end())
   {
     clause_type= cur_clause->type;
@@ -1014,9 +1020,10 @@ bool contains(const const_string & str, const const_string & infix)
 ///////////////////////////////////////////////////////////////////////////////
 MYX_Q_TYPE Query::calc_query_type()
 {
-  const_string sql(m_sql.c_str(),m_sql.length());
-  const_string masked_sql(m_masked_sql.c_str(),m_masked_sql.length());
-  return  contains(masked_sql,  CONST_STR("UNION"))   ? MYX_QT_UNION    :
+  const char * pos= m_masked_sql.c_str();
+  for ( ; pos!=&*m_masked_sql.end() && isspace(*pos); pos++);  
+  const_string sql(pos,m_masked_sql.length() - (pos-m_masked_sql.c_str()));
+  return  contains(sql,         CONST_STR("UNION"))   ? MYX_QT_UNION    :
           start_with(sql,       CONST_STR("SELECT"))  ? MYX_QT_SELECT   :
           start_with(sql,       CONST_STR("UPDATE"))  ? MYX_QT_UPDATE   :
           start_with(sql,       CONST_STR("INSERT"))  ? MYX_QT_INSERT   :
@@ -1026,6 +1033,10 @@ MYX_Q_TYPE Query::calc_query_type()
           start_with(sql,       CONST_STR("DESC"))    ? MYX_QT_DESCRIBE :
           start_with(sql,       CONST_STR("EXPLAIN")) ? MYX_QT_EXPLAIN  :
           start_with(sql,       CONST_STR("CALL"))    ? MYX_QT_CALL     :
+          start_with(sql,       CONST_STR("CHECK"))   ? MYX_QT_CHECK    :
+          start_with(sql,       CONST_STR("ANALYZE")) ? MYX_QT_ANALYZE  :
+          start_with(sql,       CONST_STR("REPAIR"))  ? MYX_QT_REPAIR   :
+          start_with(sql,       CONST_STR("OPTIMIZE"))? MYX_QT_OPTIMIZE :
           sql==CONST_STR("")                          ? MYX_QT_EMPTY    :
                                                         MYX_QT_UNKNOWN;
 }
@@ -1121,21 +1132,26 @@ Query * s_query_get_query_tables(MYSQL *mysql, Query *query,
   const MYX_Q_CLAUSE * q_clause= query->get_clause(c_type);
   if (q_clause)
   {
-    char * table_clause= g_strdup_printf(",%s", q_clause->clause);
+    size_t q_len= q_clause->start_index - 1;
+    const char * start= query->m_sql_qithout_comments.c_str() + q_len;
+    char * masked_start= (char*)query->m_masked_sql.c_str() + q_len;
+    char old_masked_start= *masked_start;
+    *masked_start= ',';
+    size_t clause_size= q_clause->end_index - q_clause->start_index + 1;
     const char *error_str;
     int erroffset, matched[60];
     pcre * pcre_exp= pcre_compile(query_table_regexp_mask, PCRE_CASELESS,
                                   &error_str, &erroffset, NULL);
+    
     if (pcre_exp)
     {
       int offset= 0, rc;
-      while ((rc= pcre_exec(pcre_exp, NULL, table_clause, 
-                            (int)strlen(table_clause), offset, 
-                            0, matched, sizeof(matched)/sizeof(*matched)))>=0)
+      while ((rc= pcre_exec(pcre_exp,NULL,masked_start,(int)clause_size,offset,
+                            0,matched,sizeof(matched)/sizeof(*matched)))>=0)
       {
         const char *table_name, *alias;
-        pcre_get_substring(table_clause, matched, rc, 2, &table_name);
-        pcre_get_substring(table_clause, matched, rc, 3, &alias);
+        pcre_get_substring(start, matched, rc, 2, &table_name);
+        pcre_get_substring(start, matched, rc, 3, &alias);
 
         query->m_tables.push_back(MYX_Q_TABLE());
         init_table(mysql,&query->m_tables.back(),table_name,alias);
@@ -1149,7 +1165,7 @@ Query * s_query_get_query_tables(MYSQL *mysql, Query *query,
       }
       pcre_free(pcre_exp);
     }
-    g_free(table_clause);
+    *masked_start= old_masked_start;
   }
   return query;
 }
@@ -1311,8 +1327,7 @@ const char * s_get_table_charset(MYSQL * mysql, MYX_Q_TABLE * q_table)
     const char *err;
     
     G_string masked= g_strdup(q.Create_stmt.get_value());
-    mask_out_string(masked, '\'', '\'', 'x');
-
+    mask_quotas(masked);
     pcre * pcre_exp= pcre_compile("DEFAULT CHARSET=(\\w+)",
                                   0, &err, &erroff, NULL);
     if ((sc= pcre_exec(pcre_exp, NULL, masked, (int)strlen(masked),
@@ -1400,32 +1415,6 @@ MYX_QUERY * query_get_query_columns(MYX_QUERY * query)
   return s_query_get_query_columns((Query*)query);
 }
 
-///////////////////////////////////////////////////////////////////////////////
-char ** split_to_columns(char * clause)
-{
-  //mask strings and () out, so we have real , seperators left
-  G_string masked_clause= g_strdup(clause);
-  mask_out_string(masked_clause, '\'', '\'', 'x');
-  mask_out_string(masked_clause, '(',  ')',  'x');
-
-  //now split the clause by ,
-  char ** columns= g_strsplit(masked_clause, ",", 0);
-  if (columns)
-  {
-    //now, replace masked columns with real columns
-    const char * real_column_pos= clause;
-    for(char ** masked_column= columns; *masked_column; masked_column++)
-    {
-      size_t column_len= strlen(*masked_column);
-      *masked_column= strncpy(*masked_column,real_column_pos,column_len);
-      real_column_pos+= column_len + 1;
-      *masked_column= g_strstrip(*masked_column);
-    }
-  }
-
-  return columns;
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 //now use the following regex to get the column infos
 //(((\w*)\.)?(\*))|(((\w*)\.)+(\w+)(\s+as\s+(\w+))?)|((.*)\s+as\s+(\w+))|(.*)
@@ -1453,11 +1442,11 @@ const char * query_column_regexp_mask=
 ///////////////////////////////////////////////////////////////////////////////
 // table_alias.* or *
 // "((("RE_NAME")\\.)?(\\*))|"
-void add_regexp_columns_with_asteric(Query *query, 
-                                     char ** columns, int rc, int * matched)
+void add_regexp_columns_with_asteric(Query *query, const char * columns,
+                                     int rc, int * matched)
 {
-  PCRE_sub_string table_alias (*columns, matched, rc, 3);
-  PCRE_sub_string column      (*columns, matched, rc, 4);
+  PCRE_sub_string table_alias (columns, matched, rc, 3);
+  PCRE_sub_string column      (columns, matched, rc, 4);
 
   G_string ta= unquote_identifier(g_strdup(table_alias));
 
@@ -1468,12 +1457,12 @@ void add_regexp_columns_with_asteric(Query *query,
 ///////////////////////////////////////////////////////////////////////////////
 // [table_alias.]column_name as column_alias
 // "((.*)\\s+as\\s+(\\w+))|"
-void add_regexp_columns_with_as(Query *query, 
-                                char ** columns, int rc, int * matched)
+void add_regexp_columns_with_as(Query *query, const char * columns,
+                                int rc, int * matched)
 {
-  PCRE_sub_string table_alias (*columns, matched, rc, 7);
-  PCRE_sub_string column      (*columns, matched, rc, 8);
-  PCRE_sub_string column_alias(*columns, matched, rc, 10);
+  PCRE_sub_string table_alias (columns, matched, rc, 7);
+  PCRE_sub_string column      (columns, matched, rc, 8);
+  PCRE_sub_string column_alias(columns, matched, rc, 10);
 
   G_string ta=         unquote_identifier(g_strdup(table_alias));
   G_string column_unq= unquote_identifier(g_strdup(column));
@@ -1484,11 +1473,11 @@ void add_regexp_columns_with_as(Query *query,
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-void add_regexp_columns_v_9(Query *query, 
-                            char ** columns, int rc, int * matched)
+void add_regexp_columns_v_9(Query *query, const char * columns,
+                            int rc, int * matched)
 {
-  PCRE_sub_string table_alias (*columns, matched, rc, 7);
-  PCRE_sub_string column      (*columns, matched, rc, 8);
+  PCRE_sub_string table_alias (columns, matched, rc, 7);
+  PCRE_sub_string column      (columns, matched, rc, 8);
 
   G_string ta=         unquote_identifier(g_strdup(table_alias));
   G_string column_unq= unquote_identifier(g_strdup(column));
@@ -1500,10 +1489,10 @@ void add_regexp_columns_v_9(Query *query,
 
 ///////////////////////////////////////////////////////////////////////////////
 void add_regexp_columns_v_14(Query *query, 
-                             char ** columns, int rc, int * matched)
+                             const char * columns, int rc, int * matched)
 {
-  PCRE_sub_string column        (*columns, matched, rc, 12);
-  PCRE_sub_string column_alias  (*columns, matched, rc, 13);
+  PCRE_sub_string column        (columns, matched, rc, 12);
+  PCRE_sub_string column_alias  (columns, matched, rc, 13);
 
   G_string column_unq= unquote_identifier(g_strdup(column));
   query_add_table_column(query, NULL, column_unq, column_alias);
@@ -1512,54 +1501,120 @@ void add_regexp_columns_v_14(Query *query,
 ///////////////////////////////////////////////////////////////////////////////
 // (.*)
 void add_regexp_columns_v_15(Query *query, 
-                             char ** columns, int rc, int * matched)
+                             const char * columns, int rc, int * matched)
 {
-  PCRE_sub_string column(*columns, matched, rc, 14);
+  PCRE_sub_string column(columns, matched, rc, 14);
   G_string column_unq= unquote_identifier(g_strdup(column));
   query_add_table_column(query, NULL, column_unq, column_unq);
 }
 
+///////////////////////////////////////////////////////////////////////////////
+struct line_delimiter_enumerator
+{
+  const char * begin;
+  const char * end;
+  const char * pos;
+  char delimiter;
+
+  line_delimiter_enumerator(const const_string & text, char delimiter= ',')
+  {
+    this->delimiter= delimiter;
+    this->begin= text.data();
+    this->end= begin + text.length();
+    this->pos= begin;
+    for ( ; pos!=end && *pos!=delimiter; pos++);
+  }
+
+  const_string operator * ()
+  {
+    return const_string(begin,pos-begin);
+  }
+
+  bool is_valid()
+  {
+    return pos!=0;
+  }
+
+  void operator ++ ()
+  {
+    if (pos==end)
+    {
+      pos= 0;
+      begin= 0;
+    }
+    else
+    {
+      pos++;
+      begin= pos;
+      for (; pos!=end && *pos!=delimiter; pos++);
+    }
+  }
+};
+///////////////////////////////////////////////////////////////////////////////
+
+///////////////////////////////////////////////////////////////////////////////
+void strip(const_string * str)
+{
+  const char *pos;
+  for (pos= str->begin(); 
+       pos!=str->end() && isspace(*pos); pos++);
+  *str= const_string(pos,str->length()-(pos-str->begin()));
+  if (!str->empty())
+  {
+    for (pos= str->end()-1;
+         pos!=str->begin() && isspace(*pos); pos--);
+    *str= const_string(str->begin(),pos-str->begin()+1);
+  }
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 Query * s_query_get_query_columns(Query *query)
 {
   const MYX_Q_CLAUSE * select_clause;
-  char ** columns;
   const char *error_str;
   int erroffset;
   pcre * pcre_exp;
 
-  if ((select_clause= query->get_clause(MYX_QCT_SELECT_CLAUSE)) &&
-      (columns= split_to_columns(select_clause->clause)))
-  {
-    if ((pcre_exp= pcre_compile(query_column_regexp_mask, PCRE_CASELESS,
+  if (!(select_clause= query->get_clause(MYX_QCT_SELECT_CLAUSE)))
+    return query;
+
+  size_t q_len= select_clause->start_index - 1;
+  const char * start= query->m_sql_qithout_comments.c_str() + q_len;
+  char * masked_start= (char*)query->m_masked_sql.c_str() + q_len;
+  const_string sclause(masked_start,
+                       select_clause->end_index - select_clause->start_index);
+
+  if ((pcre_exp= pcre_compile(query_column_regexp_mask, PCRE_CASELESS,
                               &error_str, &erroffset, NULL)))
-    {
-      for (char ** column= columns; *column; column++)
-      {                                                 // Analyse every column
-        int matched[60];
-        int rc= pcre_exec(pcre_exp, NULL, *column, (int)strlen(*column), 0, 0,
-                          matched, sizeof(matched)/sizeof(*matched));
-        switch (rc)
-        {
-        case 5: // table_alias.* or * -> "((("RE_NAME")\\.)?(\\*))|"
-              add_regexp_columns_with_asteric(query,column,rc,matched); break;
-
-        case 9:
-                add_regexp_columns_v_9(query,column,rc,matched); break;
-
-        case 11: // [table_alias.]column_name as column_alias -> 
-                add_regexp_columns_with_as(query,column,rc,matched); break;
-
-        case 14:
-                add_regexp_columns_v_14(query,column,rc,matched); break;
-
-        case 15: // (.*)
-                add_regexp_columns_v_15(query,column,rc,matched); break;
-        }
+  {
+    for (line_delimiter_enumerator e(sclause);e.is_valid(); ++e)
+    {                                                 // Analyse every column
+      const_string masked_col= *e;
+      strip(&masked_col);
+      const_string col(start+(masked_col.begin()-masked_start),
+                       masked_col.length());
+      char old_end= *col.end();
+      *(char*)col.end()= 0;
+      int matched[60];
+      int rc= pcre_exec(pcre_exp, NULL, masked_col.data(),
+                        (int)masked_col.length(), 0,0,
+                        matched, sizeof(matched)/sizeof(*matched));
+      switch (rc)
+      {
+      case 5: // table_alias.* or * -> "((("RE_NAME")\\.)?(\\*))|"
+           add_regexp_columns_with_asteric(query,col.data(),rc,matched); break;
+      case 9:
+                    add_regexp_columns_v_9(query,col.data(),rc,matched); break;
+      case 11: // [table_alias.]column_name as column_alias -> 
+                add_regexp_columns_with_as(query,col.data(),rc,matched); break;
+      case 14:
+                   add_regexp_columns_v_14(query,col.data(),rc,matched); break;
+      case 15: // (.*)
+                   add_regexp_columns_v_15(query,col.data(),rc,matched); break;
       }
-      pcre_free(pcre_exp);
+      *(char*)col.end()= old_end;
     }
-    g_strfreev(columns);
+    pcre_free(pcre_exp);
   }
   return query;
 }
@@ -1776,7 +1831,7 @@ MYX_QUERY * query_set_params(MYX_QUERY * query, MYX_STRINGLIST * parameters)
 static char *strdup_until_semicolon(const char * sql, int * had_semicolon)
 {
   char *s= g_strdup(sql);
-  char *end= s+strlen(s);
+  char *end= s+strlen(s)-1;
 
   while (end > s && isspace(*end) && *end!=';') end--;
   if (*end==';')
@@ -1847,7 +1902,7 @@ void myx_query_add_column_to_clause(Query & query,
     }
   }
 
-  *result << head << infix << alias << dot << column;
+  *result << head << infix << alias << dot << CONST_STR("`") << column << CONST_STR("`");
   *cursor_pos= (int)result->length();
   const_string tail= get_right(query.m_sql, clause->end_index);
   *result << prepare_tail_seperator(tail,clause) << tail;
@@ -1995,13 +2050,15 @@ char * myx_query_add_column_to_sql(MYSQL * mysql,
   else // If not found, add clause
   {
     std::string new_sql;
+    char *s;
     myx_query_add_column_after_clause(query,
                                       const_string(schema,strlen(schema)),
                                       const_string(table,strlen(table)),
                                       clause_type,&new_sql);
-    return myx_query_add_column_to_sql(mysql,catalog,schema,table,column,
+    s= myx_query_add_column_to_sql(mysql,catalog,schema,table,column,
                                        (char*)new_sql.c_str(),
                                        clause_type,cursor_pos);
+    return s;
   }
 }
 
@@ -2036,6 +2093,27 @@ char * myx_query_build_from_clauses(MYX_QUERY *query)
   return sql;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+size_t copy_utf8_symbol(char * dst, const char * src, size_t max_size)
+{
+  size_t len= strlen(src);
+  if (max_size<len)
+    len= max_size;
+  unsigned char *invalid_chars= 0;
+  for (size_t size= 1; size<max_size; size++)
+  {
+    if (g_utf8_validate(src,(gssize)size,(const gchar**)&invalid_chars))
+    {
+      memcpy(dst,src,size);
+      dst[size]= 0;
+      return size;
+    }
+  }
+  dst[0]= 'x';
+  dst[1]= 0;
+  return 1;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // Returns the alias of the first query-table that is still empty
 // if it is not empty, it returns the alias of the new_tablename
@@ -2043,8 +2121,9 @@ char * myx_query_get_table_alias(Query *query, const char * new_tablename)
 {
   char alias_project[40];
 
-  alias_project[0]= new_tablename[0];
-  alias_project[1]= 0;
+  size_t first_symbol_size= copy_utf8_symbol(alias_project,
+                                             new_tablename,
+                                             sizeof(alias_project)-1);
   MYX_Q_TABLE *q_table1= query->get_table_by_alias(alias_project);
   if (!q_table1)
     return g_strdup(alias_project);
@@ -2053,15 +2132,21 @@ char * myx_query_get_table_alias(Query *query, const char * new_tablename)
   for (pos1= q_table1->name, pos2= new_tablename;
        *pos1 && *pos2 && *pos1==*pos2; pos1++, pos2++);
 
-  alias_project[1]= *pos2 ? *pos2 : new_tablename[1];
-  alias_project[2]= 0;
+  size_t max_size2= sizeof(alias_project)-first_symbol_size-1;
+  size_t second_symbol_size= *pos2
+                  ? copy_utf8_symbol(alias_project+first_symbol_size,pos2,
+                                     max_size2)
+                  : copy_utf8_symbol(alias_project+first_symbol_size,
+                                     new_tablename+first_symbol_size,
+                                     max_size2);
+
   MYX_Q_TABLE *q_table2= query->get_table_by_alias(alias_project);
   if (!q_table2)
     return g_strdup(alias_project);
 
-  for (size_t i= 0; i<100; i++)
+  for (unsigned int i= 0; i<100; i++)
   {
-    sprintf(alias_project+1,"%u",i);
+    sprintf(alias_project+first_symbol_size,"%u",i);
     MYX_Q_TABLE *q_tablei= query->get_table_by_alias(alias_project);
     if (!q_tablei)
       return g_strdup(alias_project);
@@ -2395,7 +2480,7 @@ void query_add_table_to_select(MYSQL *mysql, const const_string & prepared_sql,
 const_string prepare_sql_to_add_table(MYSQL *mysql, const char * sql,
                                       const char *table)
 {
-  char * cprepared_sql= str_trim(g_strdup(sql));
+  char * cprepared_sql= utf8_str_trim(g_strdup(sql));
 
   const_string prepared_sql(cprepared_sql,strlen(cprepared_sql));
   size_t new_length= prepared_sql.length()-1;
