@@ -23,8 +23,22 @@ int main()
         {
          *pos= 0;
         }
+#ifndef _WINDOWS_
+	{
+	  for (char * pos= filename; *pos; pos++)
+	  {
+	    if  (*pos=='\\')
+	      *pos= '/';
+	  }
+	}
+#endif
       }
       const char * charset= myx_detect_charset_from_sql_file(filename);
+      for (char * pos= filename; *pos; pos++)
+      {
+	if  (*pos=='\\' || *pos=='/')
+	  *pos= '|';
+      }
       printf("charset in file \"%s\" is \"%s\"\n",filename,charset);
       g_free((gpointer)charset);
     }
