@@ -30,6 +30,10 @@
 
 #include <math.h>
 
+#ifdef _WINDOWS
+#include <Windows.h>
+#endif
+
 
 typedef struct {
   MYX_GRT *grt;
@@ -226,7 +230,7 @@ static int l_cd(lua_State *l)
 
   new_path= get_abspath(ctx, path);
 
-  value= myx_grt_dict_set_value_by_path(ctx->grt->root, new_path);
+  value= myx_grt_dict_item_get_by_path(ctx->grt->root, new_path);
 
   if (!value)
   {
@@ -257,7 +261,7 @@ static int l_ls(lua_State *l)
     return 0;
   
   fpath= get_abspath(ctx, path);
-  value= myx_grt_dict_set_value_by_path(ctx->grt->root, fpath);
+  value= myx_grt_dict_item_get_by_path(ctx->grt->root, fpath);
   g_free(fpath);
   if (!value)
     luaL_error(l, "Invalid path");
@@ -277,10 +281,10 @@ static int l_ls(lua_State *l)
     
     for (i= 0; i < value->value.l->items_num; i++)
     {
-      MYX_GRT_VALUE *item= myx_grt_list_get_item(value, i);
+      MYX_GRT_VALUE *item= myx_grt_list_item_get(value, i);
       const char *name;
       
-      name= myx_grt_dict_item_as_string(item, "name");
+      name= myx_grt_dict_item_get_as_string(item, "name");
       if (name)
         myx_grt_printf(ctx->grt, "%s"NL, name);
       else
@@ -308,7 +312,39 @@ static int l_pwd(lua_State *l)
 }
 
 
-static int l_getobj(lua_State *l)
+static int l_get_child(lua_State *l)
+{
+  LUA_CONTEXT *ctx= myx_lua_get_ctx(l);
+  MYX_GRT_VALUE *value= NULL;
+  MYX_GRT_VALUE *dict= NULL;
+  const char *path;
+
+  path= mlua_popstring(l);
+
+  if (lua_gettop(l) > 0)
+  {
+    dict= pop_grt_value(l);
+    if (!dict)
+      luaL_error(l, "bad GRT value in argument");
+  }
+
+  if (dict)
+  {
+    if (*path != '/')
+      luaL_error(l, "bad path for child object in dict. Must be an absolute path");
+    value= myx_grt_dict_item_get_by_path(dict, path);
+    myx_grt_value_release(dict);
+  }
+  if (!value)
+    luaL_error(l, "object '%s' not found", path);
+
+  myx_grt_value_retain(value);
+  myx_lua_push_wrap_grt_value(l, value);
+
+  return 1;
+}
+
+static int l_get_global(lua_State *l)
 {
   LUA_CONTEXT *ctx= myx_lua_get_ctx(l);
   MYX_GRT_VALUE *value;
@@ -328,13 +364,13 @@ static int l_getobj(lua_State *l)
   {
     if (*path != '/')
       luaL_error(l, "bad path for getobj in dict. Must be an absolute path");
-    value= myx_grt_dict_set_value_by_path(dict, path);
+    value= myx_grt_dict_item_get_by_path(dict, path);
     myx_grt_value_release(dict);
   }
   else
   {
     char *tmp= get_abspath(ctx, path);
-    value= myx_grt_dict_set_value_by_path(ctx->grt->root, tmp);
+    value= myx_grt_dict_item_get_by_path(ctx->grt->root, tmp);
     g_free(tmp);
   }
   if (!value)
@@ -347,7 +383,7 @@ static int l_getobj(lua_State *l)
 }
 
 
-static int l_setobj(lua_State *l)
+static int l_set_global(lua_State *l)
 {
   LUA_CONTEXT *ctx= myx_lua_get_ctx(l);
   MYX_GRT_VALUE *value;
@@ -379,7 +415,7 @@ static int l_setobj(lua_State *l)
       myx_grt_value_release(value);
       luaL_error(l, "bad path for setobj in dict. Must be an absolute path");
     }
-    if (myx_grt_dict_get_value_by_path(indict, path, value) < 0)
+    if (myx_grt_dict_item_set_by_path(indict, path, value) < 0)
     {
       myx_grt_value_release(indict);
       myx_grt_value_release(value);
@@ -389,16 +425,24 @@ static int l_setobj(lua_State *l)
   }
   else
   {
-    char *fpath;
-
-    fpath= get_abspath(ctx, path);
-    if (myx_grt_dict_get_value_by_path(ctx->grt->root, fpath, value) < 0)
+    if (strcmp2(path, "/") == 0)
     {
-      g_free(fpath);
+      MYX_GRT_ERROR error= myx_grt_set_root(ctx->grt, value);
       myx_grt_value_release(value);
-      luaL_error(l, "invalid path '%s'", path);
     }
-    g_free(fpath);
+    else
+    {
+      char *fpath;
+
+      fpath= get_abspath(ctx, path);
+      if (myx_grt_dict_item_set_by_path(ctx->grt->root, fpath, value) < 0)
+      {
+        g_free(fpath);
+        myx_grt_value_release(value);
+        luaL_error(l, "invalid path '%s'", path);
+      }
+      g_free(fpath);
+    }
   }
   myx_grt_value_release(value);
 
@@ -406,7 +450,7 @@ static int l_setobj(lua_State *l)
 }
 
 
-static int l_unwrap_value(lua_State *l)
+static int l_grt_value_to_lua(lua_State *l)
 {
   if (luaL_checkudata(l, -1, "MYX_GRT_VALUE"))
   {
@@ -420,6 +464,108 @@ static int l_unwrap_value(lua_State *l)
   return 1;
 }
 
+static int l_grt_value_new(lua_State *L)
+{
+  MYX_GRT *grt= myx_lua_get_grt(L);
+  const char *gstruct_name;
+  const char *name;
+  const char *_id;
+  const char *owner;
+  MYX_GRT_VALUE *value;
+
+  if (!lua_isstring(L, -1) || !lua_isstring(L, -2) || 
+    !lua_isstring(L, -3) || !lua_isstring(L, -4))
+  {
+    luaL_error(L, "invalid parameters, use {struct_name, name, _id, owner}");
+  }
+
+
+  owner= mlua_popstring(L);
+  _id= mlua_popstring(L);
+  name= mlua_popstring(L);
+  gstruct_name= mlua_popstring(L);
+
+
+  value= myx_grt_dict_new_obj(grt, gstruct_name, name, _id, owner);
+
+  //push the Grt value as Lua userdata
+  myx_lua_push_wrap_grt_value(L, value);
+
+  return 1;
+}
+
+static int l_grt_value_new_list(lua_State *L)
+{
+  MYX_GRT *grt= myx_lua_get_grt(L);
+  const char *content_type_name= NULL;
+  const char *content_struct_name= NULL;
+  MYX_GRT_VALUE *value;
+  MYX_GRT_VALUE_TYPE content_type;
+
+  if (lua_isstring(L, -1) && lua_isstring(L, -2))
+  {
+    content_struct_name= mlua_popstring(L);
+    content_type_name= mlua_popstring(L);
+  }
+  else if (lua_isstring(L, -1))
+    content_type_name= mlua_popstring(L);
+
+  if (content_type_name)
+  {
+    MYX_GRT_ERROR error;
+
+    content_type= myx_get_value_type_from_string(content_type_name, &error);
+
+    if (error != MYX_GRT_NO_ERROR)
+      luaL_error(L, "invalid content_type. Use int, real, string, list or dict");
+  }
+  else
+    content_type= MYX_ANY_VALUE;
+
+  value= myx_grt_list_new(content_type, content_struct_name);
+
+  //push the Grt value as Lua userdata
+  myx_lua_push_wrap_grt_value(L, value);
+
+  return 1;
+}
+
+static int l_grt_value_new_dict(lua_State *L)
+{
+  MYX_GRT *grt= myx_lua_get_grt(L);
+  const char *content_type_name= NULL;
+  const char *content_struct_name= NULL;
+  MYX_GRT_VALUE *value;
+  MYX_GRT_VALUE_TYPE content_type;
+
+  if (lua_isstring(L, -1) && lua_isstring(L, -2))
+  {
+    content_struct_name= mlua_popstring(L);
+    content_type_name= mlua_popstring(L);
+  }
+  else if (lua_isstring(L, -1))
+    content_type_name= mlua_popstring(L);
+
+  if (content_type_name)
+  {
+    MYX_GRT_ERROR error;
+
+    content_type= myx_get_value_type_from_string(content_type_name, &error);
+
+    if (error != MYX_GRT_NO_ERROR)
+      luaL_error(L, "invalid content_type. Use int, real, string, list or dict");
+  }
+  else
+    content_type= MYX_ANY_VALUE;
+
+  value= myx_grt_dict_new_typed(content_type, content_struct_name);
+
+  //push the Grt value as Lua userdata
+  myx_lua_push_wrap_grt_value(L, value);
+
+  return 1;
+}
+
 
 static int l_print(lua_State *L)
 {
@@ -490,7 +636,7 @@ static int l_validate(lua_State *L)
   if (argc == 2)
   {
     if (value->type == MYX_DICT_VALUE)
-      sname= myx_grt_value_struct_name(value);
+      sname= myx_grt_dict_struct_get_name(value);
   }
 
   if (sname)
@@ -520,7 +666,7 @@ static int l_validate(lua_State *L)
     return 1;
   }
 
-  if (myx_grt_value_validate_struct(grt, value, sname, strict)>0)
+  if (myx_grt_dict_struct_validate(grt, value, sname, strict)>0)
     lua_pushnumber(L, 1);
   else
     lua_pushnumber(L, 0);
@@ -546,7 +692,7 @@ static int l_getstruct(lua_State *L)
     luaL_error(L, "value must be of type list or dict");
   }
 
-  sname= myx_grt_value_struct_name(value);
+  sname= myx_grt_dict_struct_get_name(value);
 
   myx_grt_value_release(value);
   
@@ -575,9 +721,9 @@ static int l_setstruct(lua_State *L)
     luaL_error(L, "missing value");
 
   if (value->type == MYX_DICT_VALUE)
-    myx_grt_dict_assign_struct(value, sname);
+    myx_grt_dict_struct_set_name(value, sname);
   else if (value->type == MYX_LIST_VALUE)
-    myx_grt_list_assign_struct(value, sname);
+    myx_grt_list_content_set_struct_name(value, sname);
   else
   {
     myx_grt_value_release(value);
@@ -589,6 +735,53 @@ static int l_setstruct(lua_State *L)
   return 0;
 }
 
+static int l_struct_exists(lua_State *L)
+{
+  MYX_GRT *grt= myx_lua_get_grt(L);
+
+  if (!lua_isstring(L, -1))
+    luaL_error(L, "missing struct name");
+
+  if (myx_grt_struct_get(grt, mlua_popstring(L)))
+    lua_pushboolean(L, 1);
+  else
+    lua_pushboolean(L, 0);
+
+  return 1;
+}
+
+
+static int l_setcontenttype(lua_State *L)
+{
+  MYX_GRT *grt= myx_lua_get_grt(L);
+  const char *content_type_name;
+  MYX_GRT_VALUE *value;
+  MYX_GRT_VALUE_TYPE content_type;
+  MYX_GRT_ERROR error;
+
+  content_type_name= mlua_popstring(L);
+  content_type= myx_get_value_type_from_string(content_type_name, &error);
+  if (error != MYX_GRT_NO_ERROR)
+    luaL_error(L, "the content type has to be \"\", \"string\", \"int\", \"real\", \"list\" or \"dict\".");
+
+  value= pop_grt_value(L);
+  if (!value)
+    luaL_error(L, "missing value");
+
+  if (value->type == MYX_DICT_VALUE)
+    myx_grt_dict_content_set_type(value, content_type);
+  else if (value->type == MYX_LIST_VALUE)
+    myx_grt_list_content_set_type(value, content_type);
+  else
+  {
+    myx_grt_value_release(value);
+    luaL_error(L, "value must be of type list or dict");
+  }
+  
+  myx_grt_value_release(value);
+  
+  return 0;
+}
 
 static int l_run(lua_State *L)
 {
@@ -598,7 +791,7 @@ static int l_run(lua_State *L)
 
   path= mlua_popstring(L);
 
-  err= myx_grt_lua_shell_run_file(grt, path);
+  err= myx_grt_lua_shell_run_file(grt, path, 1);
   if (err == MYX_GRT_CANT_OPEN_FILE)
     luaL_error(L, "can't open the script file");
   else if (err != MYX_GRT_NO_ERROR)
@@ -607,6 +800,20 @@ static int l_run(lua_State *L)
   return 0;
 }
 
+static int l_sleep(lua_State *L)
+{
+  int ms= luaL_checkint(L, -1);
+  lua_pop(L, 1);
+
+#ifdef _WINDOWS
+  Sleep(ms);
+#else
+  usleep(ms*1000);
+#endif
+
+  return 0;
+}
+
 
 /** 
  ****************************************************************************
@@ -656,10 +863,14 @@ MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt)
   lua_register(lua, "refresh", l_refresh);
 
   // struct stuff
-  lua_register(lua, "stload", l_loadstructs);
-  lua_register(lua, "stvalidate", l_validate);
-  lua_register(lua, "stset", l_setstruct);
-  lua_register(lua, "stget", l_getstruct);
+  lua_register(lua, "structLoad", l_loadstructs);
+  lua_register(lua, "structVal", l_validate);
+  lua_register(lua, "structSet", l_setstruct);
+  lua_register(lua, "structGet", l_getstruct);
+  lua_register(lua, "structExists", l_struct_exists);
+
+  // contenttype
+  lua_register(lua, "contentTypeSet", l_setcontenttype);
 
   // module stuff
   lua_register(lua, "lsm", l_list_modules);
@@ -669,15 +880,22 @@ MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt)
   lua_register(lua, "load", l_load_value);
   lua_register(lua, "show", l_show_value);
   
-  lua_register(lua, "unwrap", l_unwrap_value);
+  // grt
+  lua_register(lua, "grt2Lua", l_grt_value_to_lua);
+  lua_register(lua, "grtNewObj", l_grt_value_new);
+  lua_register(lua, "grtNewList", l_grt_value_new_list);
+  lua_register(lua, "grtNewDict", l_grt_value_new_dict);
   
   lua_register(lua, "cd", l_cd);
   lua_register(lua, "ls", l_ls);
   lua_register(lua, "pwd", l_pwd);
-  lua_register(lua, "getobj", l_getobj);
-  lua_register(lua, "setobj", l_setobj);
+  lua_register(lua, "getGlobal", l_get_global);
+  lua_register(lua, "setGlobal", l_set_global);
+  lua_register(lua, "getChild", l_get_child);
 
   lua_register(lua, "run", l_run);
+
+  lua_register(lua, "sleep", l_sleep);
  
   myx_lua_refresh(grt, lua);
   
@@ -861,6 +1079,7 @@ static MYX_GRT_VALUE *pop_grt_value(lua_State *L)
  // MYX_GRT *grt= myx_lua_get_grt(L);
   MYX_GRT_VALUE *res= NULL;
 
+  // if this is a GRT value
   if (lua_isuserdata(L, -1))
   {
     luaL_checkudata(L, -1, "MYX_GRT_VALUE");
@@ -868,17 +1087,6 @@ static MYX_GRT_VALUE *pop_grt_value(lua_State *L)
     lua_pop(L, 1);
     myx_grt_value_retain(res);
   }
-/*  else if (lua_isstring(L, -1) && grt->root)
-  {
-    const char *path= lua_tostring(L, -1);
-    if (*path == '/')
-    {
-      lua_pop(L, 1);
-      res= myx_grt_dict_set_value_by_path(grt->root, path);
-      myx_grt_value_retain(res);
-    }
-  }
- */
 
   if (!res)
   {
@@ -904,7 +1112,7 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
       if (n - floor(n) == 0)
         value= myx_grt_value_from_int((int)n);
       else
-        value= myx_grt_value_from_double(n);
+        value= myx_grt_value_from_real(n);
       lua_pop(L, 1);
     }
     break;
@@ -914,6 +1122,7 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
     break;
   case LUA_TTABLE:
     {
+      MYX_GRT_VALUE_TYPE list_type;
       int tbl= lua_gettop(L);
       int can_be_list= 1;
       unsigned int i;
@@ -930,11 +1139,12 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
         MYX_GRT_VALUE *item_value;
 
         // handle value
-        item_value= myx_lua_pop_grt_value(L);
+        //item_value= myx_lua_pop_grt_value(L);
+        item_value= pop_grt_value(L);
 
         // handle key
         lua_pushvalue(L, -1);
-        myx_grt_dict_set_item(value, lua_tostring(L, -1), item_value);
+        myx_grt_dict_item_set_value(value, lua_tostring(L, -1), item_value);
         lua_pop(L, 1);
 
         if (lua_type(L, -1) != LUA_TNUMBER || lua_tonumber(L, -1) != nexti)
@@ -947,23 +1157,18 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
       }
       lua_pop(L, 1);
 
-      if (can_be_list)
+      if (can_be_list && (value->value.d->items_num > 0))
       {
-        MYX_GRT_VALUE_TYPE type;
-
-        if (value->value.d->items_num > 0)
+        // now we check if the values are all the same 
+        // and remember the type of the list
+        for (i= 0; i < value->value.d->items_num; i++)
         {
-          // now we check if the values are all the same and therefore 
-          // can be converted to a list
-          for (i= 0; i < value->value.d->items_num; i++)
+          if (i > 0 && list_type != value->value.d->items[i].value->type)
           {
-            if (i > 0 && type != value->value.d->items[i].value->type)
-            {
-              can_be_list= 0;
-              break;
-            }
-            type= value->value.d->items[i].value->type;
+            list_type= MYX_ANY_VALUE;
+            break;
           }
+          list_type= value->value.d->items[i].value->type;
         }
       }
 
@@ -973,7 +1178,7 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
 
         value->type= MYX_LIST_VALUE;
         value->value.l= g_new0(MYX_GRT_LIST, 1);
-        value->value.l->content_type= dict->items[0].value->type;
+        value->value.l->content_type= list_type;
         value->value.l->items_num= dict->items_num;
         value->value.l->items= g_new0(MYX_GRT_VALUE*, dict->items_num);
 
@@ -992,6 +1197,8 @@ MYX_GRT_VALUE *myx_lua_pop_grt_value(lua_State *L)
     break;
 
   case LUA_TNIL:
+    lua_pop(L, 1);
+    break;
   case LUA_TFUNCTION:
   case LUA_TLIGHTUSERDATA:
   case LUA_TUSERDATA:
@@ -1039,9 +1246,27 @@ static int myx_lua_call_grt_method(lua_State *L)
   g_return_val_if_fail(func != NULL, 0);
 
   retval= NULL;
-  retval= myx_grt_function_call(grt, func, argument, &error);
 
-  myx_lua_push_wrap_grt_value(L, retval);
+  // if there was no argument given or the argument was a single value (and not a list as required)
+  // create a new list and add the given argument to that list
+  if ((argument == NULL) || (myx_grt_value_get_type(argument) != MYX_LIST_VALUE))
+  {
+    MYX_GRT_VALUE *argument_list= myx_grt_list_new(MYX_ANY_VALUE, NULL);
+
+    if (argument)
+      myx_grt_list_item_add(argument_list, argument);
+
+    retval= myx_grt_function_call(grt, func, argument_list, &error);
+
+    myx_grt_value_release(argument_list);
+  }
+  else
+    retval= myx_grt_function_call(grt, func, argument, &error);
+
+  if (retval)
+    myx_lua_push_wrap_grt_value(L, retval);
+  else
+    lua_pushnil(L);
 
   if (argument)
     myx_grt_value_release(argument);
@@ -1125,7 +1350,7 @@ static int myx_lua_execute(MYX_GRT *grt, const char *linebuf)
  */
 void myx_grt_shell_print_welcome(MYX_GRT *grt)
 {
-  char *version= "0.1.1 pre-alpha";
+  char *version= "2.0.4 alpha";
 
   myx_grt_printf(grt, "MySQL Generic Runtime Environment %s"NL, version);
 
@@ -1158,6 +1383,96 @@ static void myx_grt_shell_show_command_help(MYX_GRT *grt, MYX_GRT_SHELL_COMMAND
   }
 }
 
+//use the following regex to detect all global paths in a function
+//((.*?)[\(\,\=]\s*)((\/(\/(.*?))\/)|(\/(.*?)))(\s*[\)\,\n\r](.*))
+// 1: cmd prefix
+// 2: -
+// 3: -
+// 4: -
+// 5: global path (if escaped //path/)
+// 6: -
+// 7: global path (if not an escaped /path)
+// 8: -
+// 9: cmd postfix
+// 10: -
+// tests:
+//   mod:func( /test/path_entry )
+//   mod:func("/test", /test/path_entry)
+//   mod:func("/test", //test/path_entry/)
+//   test= /test
+//   test= //test path/
+#define REGEX_GLOBAL_PATH_IN_FUNC "((.*?)[\\(\\,\\=]\\s*)((\\/(\\/(.*?))\\/)|(\\/(.*?)))(\\s*[\\)\\,\\n\\r](.*))"
+//#define REGEX_GLOBAL_PATH_IN_FUNC "((.*?))((\\/(\\/(.*?))\\/)|(\\/(.*?)))"
+
+static char *substitude_global_path(char *cmd)
+{
+  pcre *pcre_exp;
+  const char *error_str;
+  int erroffset;
+  int o_vector[64];
+  int rc;
+  const char *prefix;
+  const char *path;
+  const char *postfix;
+  char *value= NULL;
+
+  if(cmd && cmd[0])
+  {
+    char *txt= g_strdup(cmd);
+
+    pcre_exp= pcre_compile(REGEX_GLOBAL_PATH_IN_FUNC, PCRE_MULTILINE, &error_str, &erroffset, NULL);
+    if (pcre_exp)
+    {
+      if ( (rc= pcre_exec(pcre_exp, NULL, txt, (unsigned int)strlen(txt), 0, 
+                          0, o_vector, sizeof(o_vector)/sizeof(int)) ) > 0)
+      {
+        // get prefix, e.g. `mod:func(`
+        pcre_get_substring(txt, o_vector, rc, 1, &prefix);
+        value= g_strdup(prefix);
+        pcre_free_substring((char*)prefix);
+
+        // get escaped path, e.g. //test/path, replace it with getGlobal("/test/path")
+        // if there is no escaped path, it must be a normal path,
+        // e.g. /test/path, replace it with getGlobal("/test/path")      
+        if (pcre_get_substring(txt, o_vector, rc, 5, &path) == 0)
+          pcre_get_substring(txt, o_vector, rc, 7, &path);
+        value= str_g_append_and_free(value, g_strdup_printf("getGlobal(\"%s\")", path));
+        pcre_free_substring((char*)path);
+
+        /*if (o_vector[10] > -1)
+        {
+          pcre_get_substring(txt, o_vector, rc, 5, &path);
+          value= str_g_append_and_free(value, g_strdup_printf("getGlobal(\"%s\")", path));
+          pcre_free_substring((char*)path);
+        }
+        // get path, e.g. /test/path, replace it with getGlobal("/test/path")        
+        else
+        {
+          pcre_get_substring(txt, o_vector, rc, 7, &path);
+          value= str_g_append_and_free(value, g_strdup_printf("getGlobal(\"%s\")", path));
+          pcre_free_substring((char*)path);
+        }*/
+
+        // get postfix, e.g. `)`
+        pcre_get_substring(txt, o_vector, rc, 9, &postfix);
+        value= str_g_append(value, postfix);
+        pcre_free_substring((char*)postfix);
+
+        g_free(cmd);
+
+        //call recursive to replace all global paths
+        cmd= substitude_global_path(value);
+      }
+
+      pcre_free(pcre_exp);
+    }
+
+    g_free(txt);
+  }
+
+  return cmd;
+}
+
 /** 
  ****************************************************************************
  * @brief Shows help text pertaining to the Lua Shell.
@@ -1171,8 +1486,33 @@ static void myx_grt_shell_show_command_help(MYX_GRT *grt, MYX_GRT_SHELL_COMMAND
  */
 void myx_grt_shell_show_help(MYX_GRT *grt, const char *command)
 {
+#define BLOCK_SEPERATOR NL"--------------------------------------------------------------------------------"NL
+
+#define BLOCK_HEADER(varname, text, linechar, width) {\
+    const char *txt= text;\
+    unsigned int i;\
+    unsigned int len= (unsigned int)strlen(txt);\
+    unsigned int p= div(width, 2).quot - div(len, 2).quot;\
+    varname= g_malloc(sizeof(char)*81);\
+    for (i= 0; i < width; i++)\
+      varname[i]= linechar;\
+    varname[width]= 0;\
+    for (i= 0; i < len; i++)\
+      varname[p + i]= txt[i];\
+  }
+
   if(!command)
   {
+    char *file_funcs;
+    char *globals_vars;
+    char *grt_st;
+    //char *grt_obj;
+
+    BLOCK_HEADER(file_funcs, "FILE FUNCTIONS", '-', 80)
+    BLOCK_HEADER(globals_vars, "GLOBALS / VARIABLES", '-', 80)
+    BLOCK_HEADER(grt_st, "GRT STRUCTS", '-', 80)
+    
+
     myx_grt_printf(grt, "List of shell commands:"NL
       "Type 'help command' to get detailed information about the command. "NLNL
       "help    (\\h)    Display this help."NL
@@ -1180,22 +1520,50 @@ void myx_grt_shell_show_help(MYX_GRT *grt, const char *command)
       "quit    (\\q)    Exit the shell."NL
       "exit    (\\e)    Synonym for 'quit'."NL
       NL
-      "run filename    (\\r)           Load and execute a lua script file."NL
-      "ls [-m [module]]               List all objects of the current object."NL
-      "cd objname                     Changes the current object."NL
-      "show objname                   Display the object"NL
+      "%s"NL // FILE FUNCTIONS
       NL
+      "run filename    (\\r)           Load and execute a lua script file."NL
       "obj= load(\"filename\")          Loads an object"NL
       "save(obj, \"filename\")          Saves an object"NL
-      "setobj(path, obj)              Sets the object defined by path to the given object"NL
-      "getobj(path)                   Returns the object defined by path"NL
       NL
-      "stload(\"filename\")             Register structs from file"NL
-      "stset(obj, \"struct\")           Assigns the struct defined by structname to the object"NL
-      "stget(obj)                     Returns the struct of an object"NL
-      "stvalidate(obj, \"struct\", 0)   Returns ~= 0 if the objects corresponds to the struct"NL
-      "stvalidate(obj, 0)             Returns ~= 0 if the objects corresponds to it's assigned struct"NL
-      );
+      "%s"NL // GLOBALS / VARIABLES
+      NL
+      "ls [-m [module]]               List all child objects of the current global."NL
+      "cd path                        Changes the current global."NL
+      "show [/path | obj]             Display the global or variable"NLNL
+      "/path = obj                    Assigns the object to the global"NL
+      "//path with space/ = obj"NL
+      "setGlobal(\"/path\", obj)"NLNL
+      "obj = /path                    Assigns the global to the object"NL
+      "obj = //path with space/"NL
+      "getGlobal(\"/path\")             Assigns the global to the object"NLNL
+      "getChild(obj, \"/path\")         Returns the object's child defined by the path"NL
+      NL 
+      "lua_value= grt2Lua(grt_value)  Converts the GRT object to a lua value."NL
+      "grt_value= grtNewObj(\"struct\", \"name\", \"_id\", \"owner\")"NL
+      "                               Create a new Grt object with the given values"NL
+      "grt_value= grtNewList(\"content-type\" [, \"content-struct\"])"NL
+      "                               Create a new Grt list with the given content type"NL
+      "grt_value= grtNewDict(\"content-type\" [, \"content-struct\"])"NL
+      "                               Create a new Grt dict with the given content type"NL
+      NL
+      "%s"NL // GRT STRUCTS
+      NL
+      "structLoad(\"filename\")         Register structs from file"NL
+      "structSet(obj, \"struct\")       Assigns the struct to the object"NL
+      "structGet(obj)                 Returns the struct of an object"NL
+      "structVal(obj, \"struct\", 0)    Returns ~= 0 if the obj corresponds to the struct"NL
+      "structVal(obj, 0)              Returns ~= 0 if the objects corresponds to its "NL
+      "                               assigned struct"NL
+      "structExists(\"struct\")         Returns 1 if the struct exists"NL
+      "contentTypeSet(obj, \"type\")    Sets the content type of the given list or dict"NL,
+      file_funcs,
+      globals_vars,
+      grt_st);
+
+     g_free(file_funcs);
+     g_free(globals_vars);
+     g_free(grt_st);
   }
   else
   {
@@ -1208,6 +1576,8 @@ void myx_grt_shell_show_help(MYX_GRT *grt, const char *command)
     else if(strcmp2(command, "run")==0)
       myx_grt_shell_show_command_help(grt, MYX_GRT_SHELL_COMMAND_RUN);
   }
+
+#undef BLOCK_SEPERATOR
 }
 
 /** 
@@ -1231,14 +1601,18 @@ MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebu
   char *cmd_param;
   MYX_GRT_SHELL_COMMAND res= MYX_GRT_SHELL_COMMAND_UNKNOWN;
   char *preprocessed_cmd= NULL;
+  char *line= g_strdup(linebuf);
 
-  cmd= g_strdup(linebuf);
+  //substitude global path with getGlobal(globale path)
+  line= substitude_global_path(line);
+
+  cmd= g_strdup(line);
   cmd= str_trim(cmd);
   cmd_len= (unsigned int)strlen(cmd);
   cmd_param= NULL;
 
   //Help command
-  if((cmd_param= get_value_from_text_ex(cmd, cmd_len, "^(help|help\\;|\\\\h|\\?|\\/|\\-\\?)\\s*(\\w*)", 2)))
+  if((cmd_param= get_value_from_text_ex(cmd, cmd_len, "^(help|help\\;|\\\\h|\\?|\\-\\?)\\s*(\\w*)", 2)))
   {
     if(cmd_param[0])
       myx_grt_shell_show_help(grt, cmd_param);
@@ -1316,16 +1690,54 @@ MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebu
       res= MYX_GRT_SHELL_COMMAND_LUA;
     }
   }
-  // Automatically convert show objectname to show(getobj("objectname"))
+  // Automatically convert show to show(getobj(pwd()))
+  else if (strcmp2(cmd, "show") == 0)
+  {
+    preprocessed_cmd= g_strdup_printf("show(getobj(pwd()))"NL);
+    res= MYX_GRT_SHELL_COMMAND_LUA;
+  }
+  // Automatically convert show objectname to show(getObj("objectname"))
   else if (str_beginswith(cmd, "show "))
   {
     char *path= get_value_from_text_ex(cmd, (int)strlen(cmd), "show\\s+(.+)", 1);
 
-    if((path) && (path[0]))
+    if ((path) && (path[0]))
     {
-      preprocessed_cmd= g_strdup_printf("show(getobj(\"%s\"))"NL, path);
+      preprocessed_cmd= g_strdup_printf("show(getGlobal(\"%s\"))"NL, path);
       res= MYX_GRT_SHELL_COMMAND_LUA;
     }
+
+    g_free(path);
+  }
+  // Convert `/test/path_entry = res` to `setGlobal("/test/path_entry", res)`
+  else if (str_beginswith(cmd, "/"))
+  {
+    char *path= get_value_from_text_ex(cmd, cmd_len, "(/(.*?))\\s*=\\s*(.*)", 1);
+    char *val= get_value_from_text_ex(cmd, cmd_len, "(/(.*?))\\s*=\\s*(.*)", 3);
+
+    if (path && path[0] && val && val[0])
+    {
+      preprocessed_cmd= g_strdup_printf("setGlobal(\"%s\", %s)"NL, path, val);
+      res= MYX_GRT_SHELL_COMMAND_LUA;
+    }
+
+    g_free(path);
+    g_free(val);
+  }
+  // Convert `//test/path entry/ = res` to `setGlobal("/test/path entry", res)`
+  else if (str_beginswith(cmd, "//"))
+  {
+    char *path= get_value_from_text_ex(cmd, cmd_len, "(\\/(.*?)\\/)\\s*=\\s*(.*)", 2);
+    char *val= get_value_from_text_ex(cmd, cmd_len, "(\\/(.*?)\\/)\\s*=\\s*(.*)", 3);
+
+    if (path && path[0] && val && val[0])
+    {
+      preprocessed_cmd= g_strdup_printf("setGlobal(\"%s\", %s)"NL, path, val);
+      res= MYX_GRT_SHELL_COMMAND_LUA;
+    }
+
+    g_free(path);
+    g_free(val);
   }
   g_free(cmd);
   if(cmd_param)
@@ -1335,12 +1747,14 @@ MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebu
   if((res == MYX_GRT_SHELL_COMMAND_UNKNOWN) || (res == MYX_GRT_SHELL_COMMAND_LUA))
   {
     if(!preprocessed_cmd)
-      res= myx_lua_execute(grt, linebuf);
+      res= myx_lua_execute(grt, line);
     else
     {
       res= myx_lua_execute(grt, preprocessed_cmd);
       g_free(preprocessed_cmd);
     }
+
+    g_free(line);
     
     if (res > 0)
       return MYX_GRT_SHELL_COMMAND_UNKNOWN;
@@ -1350,6 +1764,8 @@ MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebu
       return MYX_GRT_SHELL_COMMAND_LUA;
   }
 
+  g_free(line);
+
   return res;
 }
 
@@ -1412,12 +1828,13 @@ lua_State *myx_grt_lua_shell_get_lua(MYX_GRT *grt)
  * @return < 0 if there's an error executing the file
  ****************************************************************************
  */
-int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name)
+int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name, int interactive)
 {
   int status= luaL_loadfile(grt->lshell->lua, file_name);
   int rc;
 
-  myx_grt_printf(grt, "Opening script file %s ..."NL, file_name);
+  if (interactive)
+    myx_grt_printf(grt, "Opening script file %s ..."NL, file_name);
 
   if (status)
   {
@@ -1425,7 +1842,8 @@ int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name)
     return -1;
   }
 
-  myx_grt_printf(grt, "Executing script file %s ..."NLNL, file_name);
+  if (interactive)
+    myx_grt_printf(grt, "Executing script file %s ..."NLNL, file_name);
 
   status= lua_pcall(grt->lshell->lua, 0, LUA_MULTRET, 0);
   if (status)
@@ -1436,7 +1854,7 @@ int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name)
   else
     rc= 0;
 
-  if (rc == 0)
+  if ((rc == 0) && (interactive))
     myx_grt_printf(grt, "\nExecution finished."NL);
 
   return rc;
