@@ -121,7 +121,9 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
     ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_press_event().connect(SigC::slot(*this,&MGConnectDialog::on_key_pressed));
     ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_key_release_event().connect(SigC::slot(*this,&MGConnectDialog::on_key_released));
   }
-  
+
+  ((Gtk::Window*)_xml->get_widget("connect_dialog"))->signal_delete_event().connect(SigC::slot(*this,&MGConnectDialog::on_window_close));
+
   _pick_schema= (flags&MYG_FLAG_PICK_SCHEMA)!=0;
 
   /* enable history in text entries */
@@ -611,6 +613,13 @@ void MGConnectDialog::preferences_closed()
 }
 
 
+bool MGConnectDialog::on_window_close(GdkEventAny *ev)
+{
+  on_cancel_button_clicked();
+  return true;
+}
+
+
 void MGConnectDialog::open_connection_editor()
 {
   // select back the original item
