@@ -4,6 +4,19 @@
 
 #include <myx_public_interface.h>
 
+
+@class MConnectionPanel;
+
+@protocol MConnectionPanelDelegate
+- (void)connectionPanel:(MConnectionPanel*)panel 
+               finished:(MYSQL*)mysql
+                   info:(MYX_USER_CONNECTION*)conn;
+
+- (void)connectionPanel:(MConnectionPanel*)panel 
+                aborted:(BOOL)status;
+@end
+
+
 @interface MConnectionPanel : NSWindowController
 {
   @protected
@@ -16,13 +29,25 @@
     IBOutlet NSSecureTextField *password;
     IBOutlet NSButton *pingButton;
     IBOutlet NSTextView *pingText;
+    IBOutlet id pingTextContainer;
     IBOutlet NSTextField *port;
     IBOutlet NSStepper *portSpin;
     IBOutlet NSProgressIndicator *progressIndicator;
     IBOutlet NSTextField *username;
     IBOutlet NSView *extraView;
+    IBOutlet NSImageView *topImage;
+	IBOutlet NSButton *compress;
+	IBOutlet NSTextField *schema;
+
+    IBOutlet NSTextField *socketText;
+
+    id<MConnectionPanelDelegate> _delegate;
+
+    int _lastSelectedConnection;
     
-    id _delegate;
+    NSPipe *pingPipe;
+    NSTask *pingTask;
+    NSFileHandle *pingPipeHandle;
     
     MYSQL *_mysql;
     BOOL _connecting;
@@ -31,6 +56,9 @@
     MYX_USER_CONNECTIONS *_connections;
     MYX_USER_CONNECTION _new_connection;
 }
+
++ (NSString*)representationForConnection:(MYX_USER_CONNECTION*)conn;
+
 - (IBAction)cancel:(id)sender;
 - (IBAction)closeErrorPanel:(id)sender;
 - (IBAction)connect:(id)sender;
@@ -38,10 +66,15 @@
 - (IBAction)pingHost:(id)sender;
 - (IBAction)toggleDetails:(id)sender;
 
-- (id)initWithConnectionsFile:(NSString*)file; //
+- (id)initWithConnectionsFile:(NSString*)file;
+
+- (void)setEditsSchema;
+
+- (void)setHeaderImage:(NSImage*)image;
 
 - (void)setDelegate:(id)deleg;
 
 - (void)show;
+- (void)showAndConnectTo:(NSString*)connection;
 
 @end
