@@ -10,6 +10,12 @@ MYX_USER_CONNECTION *myx_copy_user_connection(MYX_USER_CONNECTION *info);
 
 NSString *myx_get_available_filename(NSString *directory, NSString *base, NSString *suffix);
 
+int myx_same_user_connections(MYX_USER_CONNECTION *conn1,
+                              MYX_USER_CONNECTION *conn2);
+
+
+const char *myx_get_option_value(const char **list, unsigned int count, const char *option);
+
 #define NSStr(s) [NSString stringWithUTF8String:(s)?:""]
 
 #endif
